"""
obo.py
Provides lightweight parser of OBO ontology files. Uses an in-memory, or
optionally on-disk, SQLite3 db to handle reasoning, and can reason arbitrarily
over given predicate relationships.
"""
import sqlite3
from collections import defaultdict
import networkx as nx
import uuid
import os
from lib.common.os import cmd_run
from collections import OrderedDict
from lib.common.fileIO import tsv_file_dict_parse
#from orderedset import OrderedSet


GRAPH_SCHEMA_SQL = """
CREATE TABLE graph(subject, predicate, object,
    unique(subject, predicate, object) on conflict replace);
"""

INSERT_GRAPH_SQL = """INSERT INTO graph(subject, predicate, object)
VALUES(?, ?, ?)"""

DESC_SQL = """SELECT subject FROM graph WHERE predicate like ?
AND object like ?"""

ASC_SQL = """SELECT object FROM graph WHERE predicate like ?
AND subject like ?"""

MULTIVALUE = ['is_a', 'relationship']

ROOT_NODE = {"GO:0005575":"cellular component", "GO:0003674":"molecular function", "GO:0008150":"biological process"}


def _parse(line):
    """Returns a line split on a ': ' as a key/value."""
    if ':' in line:
        key, val = line.split(': ', 1)
        val = val.strip('\n ')
        return key, val
    else:
        return line, None


def _strip_comments(string):
    """Removes comments (text after ' ! ')."""
    if ' ! ' in string:
        val = string.split('!', 1)
        val = val[0].strip()
        return val
    else:
        return string


def parse_obo(obofile):
    """Yields stanzas as encountered in OBO file as dicts."""
    stanza = None
    kvals = defaultdict(list)

    with open(obofile, 'r') as inf:

        for line in inf:
            if stanza:
                yield stanza
            key, val = _parse(line)
            if val:
                kvals[key].append(_strip_comments(val))
            elif 'id' in kvals:
                for key in kvals:
                    val = kvals[key]
                    kvals[key] = val[0] if (len(val) == 1
                        and key not in MULTIVALUE) else val
                stanza = kvals
                kvals = defaultdict(list)
            else:
                kvals = defaultdict(list)


def _parse_relationship(string):
    """Parses the values from the relationship key.
    Ex: 'part_of GO:0001234 ! some go term' => ('part_of', 'GO:0001234')"""
    string = _strip_comments(string)
    pred, obj = string.split(' ', 1)
    return pred.strip(), obj.strip()


class Obo:
    """Ontology created from an OBO file specified during construction. Use
    Obo.query() to reason over relationships, and Obo.show_relations() to see
    available relationships.
    """
    conn = None
    stanzas = None

    def __init__(self, obofile):
        """Creates a new ontology based off a given OBO file."""
        self._createdb()
        self.stanzas = {}
        
        for stanza in parse_obo(obofile):
            self.stanzas[stanza['id']] = stanza
            self._ins_stanza(stanza)

    def _createdb(self, dbname=':memory:'):
        self.conn = sqlite3.connect(dbname)
        self.conn.execute(GRAPH_SCHEMA_SQL)

    def _ins_stanza(self, stanza):
        if 'is_obsolete' in stanza and stanza['is_obsolete']:
                return
        if 'is_a' in stanza:
            for related in stanza['is_a']:
                self.conn.execute(INSERT_GRAPH_SQL,
                    (stanza['id'], 'is_a', related))
        if 'relationship' in stanza:
            for related in stanza['relationship']:
                relationship, target = _parse_relationship(related)
                self.conn.execute(INSERT_GRAPH_SQL,
                    (stanza['id'], relationship, target))

    def _traverse_tree(self):
        pass

    def get(self, _id):
        return self.stanzas.get(_id, None)

    def find_children(self, _id, expand=False):
        """Returns children (using 'is_a' relation) of given id.
        If expand=True, returns all [great,...] grandchildren."""
        return self.query(_id, 'is_a', expand=expand)

    def find_parents(self, _id, expand=False):
        """Returns parents (using 'is_a' relation) of given id.
        If expand=True, returns all [great,...] grandparents."""
        return self.query(_id, 'is_a', desc=False, expand=expand)

    def most_recent_common_ancestor(self, _id1, _id2, predicate):
        p1 = self.query(_id1, predicate, desc=False, expand=True)
        p2 = self.query(_id2, predicate, desc=False, expand=True)
        for i, path in enumerate(p1):
            pass

    def query(self, _id, predicate, desc=True, expand=False, named=False):
        """Returns all the descendants of a given predicate relationship, or
        if desc=False, returns ascendants. If expand=True, traverses up or
        down graph to terminal nodes."""
        related = []
        sql = DESC_SQL if desc else ASC_SQL
        cur = self.conn.execute(sql, (predicate, _id))
        rset = cur.fetchall()
        cur.close()
        for relative in rset:
            path = []
            path.append((relative[0], self.stanzas[relative[0]]))
            if expand:
                for sub in self.query(relative[0], predicate, desc, expand):
                    path.append((sub[0], self.stanzas[sub[0]]))
            related.extend(path)
        return related

    def relationships(self):
        """Shows all the relationships in the graph."""
        cur = self.conn.execute("select distinct predicate from graph")
        rset = [x[0] for x in cur.fetchall()]
        cur.close()
        return rset

    def get_all_path(self, _id):

        parents_dict = self.find_parents(_id, expand=True)

        all_node_dict = {i[0]:i[1] for i in parents_dict}
        all_node_dict[_id] = self.get(_id)

        edge_list = []
        for go_id in all_node_dict:
            is_a_list = all_node_dict[go_id]['is_a']
            for p_id in is_a_list:
                edge_list.append((go_id,p_id))

        edge_list = list(set(edge_list))

        DG = nx.DiGraph()

        for go_id, p_id in edge_list:
            DG.add_edge(go_id, p_id)

        output_dict = {}
        for target_node in ROOT_NODE:
            output_dict[target_node] = list(nx.all_simple_paths(DG, source=_id, target=target_node))

        return output_dict

def GO_enrichment_by_topGO(foreground_gene_list, background_GO_dict, Ontology='BP'):
    tmp_prefix = "/tmp/" + uuid.uuid1().hex

    foreground_gene_file = tmp_prefix + ".id"
    foreground_gene_list = list(set(foreground_gene_list))
    with open(foreground_gene_file, 'w') as f:
        for i in foreground_gene_list:
            f.write("%s\n" % i)
    
    background_GO_file = tmp_prefix + ".go.base"
    with open(background_GO_file, 'w') as f:
        for i in background_GO_dict:
            background_GO_dict[i] = list(set(background_GO_dict[i]))
            f.write("%s\t%s\n" % (i, ", ".join(background_GO_dict[i])))

    topgo_r_script_file = os.path.split(os.path.realpath(__file__))[0] + "/TopGO.r"
    output_file = tmp_prefix + ".out"

    try:
        cmd_string = "Rscript %s %s %s %s %s" % (topgo_r_script_file, foreground_gene_file, background_GO_file, output_file, Ontology)
        cmd_run(cmd_string, silence=True)

        file_info_dict = tsv_file_dict_parse(output_file, ignore_head_num=1, fieldnames=['Rank', 'GO_id', 'Term', 'Annotated', 'Significant', 'Expected', 'Rank in classic', 'classic', 'KS', 'weight'])

        output_dict = OrderedDict()
        for i in file_info_dict:
            GO_id = file_info_dict[i]['GO_id']
            Rank_in_weight = int(file_info_dict[i]['Rank'])
            Term = file_info_dict[i]['Term']
            Annotated = int(file_info_dict[i]['Annotated'])
            Significant = int(file_info_dict[i]['Significant'])
            Expected = float(file_info_dict[i]['Expected'])
            Rank_in_classic = int(file_info_dict[i]['Rank in classic'])
            if file_info_dict[i]['classic'] == '< 1e-30':
                classic = 1e-35
            else:
                classic = float(file_info_dict[i]['classic'])
            KS = float(file_info_dict[i]['KS'])
            weight = float(file_info_dict[i]['weight'])

            output_dict[GO_id] = {
                "GO_id":GO_id,
                "Rank_in_weight":Rank_in_weight,
                "Term":Term,
                "Annotated":Annotated,
                "Significant":Significant,
                "Expected":Expected,
                "Rank_in_classic":Rank_in_classic,
                "classic":classic,
                "KS":KS,
                "weight":weight,
            }
    except:
        pass

    os.remove(foreground_gene_file)
    os.remove(background_GO_file)
    os.remove(output_file)

    return output_dict

def retrieve_gene_in_GO_by_topGO(GO_list, background_GO_dict, Ontology='BP'):
    tmp_prefix = "/tmp/" + uuid.uuid1().hex

    GO_list_file = tmp_prefix + ".go.id"
    with open(GO_list_file, 'w') as f:
        for i in list(set(GO_list)):
            f.write("%s\n" % i)

    foreground_gene_file = tmp_prefix + ".id"
    foreground_gene_list = list(background_GO_dict.keys())[:5]
    with open(foreground_gene_file, 'w') as f:
        for i in foreground_gene_list:
            f.write("%s\n" % i)
    
    background_GO_file = tmp_prefix + ".go.base"
    with open(background_GO_file, 'w') as f:
        for i in background_GO_dict:
            background_GO_dict[i] = list(set(background_GO_dict[i]))
            f.write("%s\t%s\n" % (i, ", ".join(background_GO_dict[i])))

    topgo_r_retrivere_script_file = os.path.split(os.path.realpath(__file__))[0] + "/TopGO_retrieve.r"
    output_file = tmp_prefix + ".out"

    try:
        cmd_string = "Rscript %s %s %s %s %s %s" % (topgo_r_retrivere_script_file, GO_list_file, foreground_gene_file, background_GO_file, output_file, Ontology)
        cmd_run(cmd_string, silence=True)

        
        file_info_dict = tsv_file_dict_parse(output_file, fieldnames=['go_id', 'g_id'], seq=' ')

        output_dict = {}
        for i in file_info_dict:
            GO_id = file_info_dict[i]['go_id']
            g_id = file_info_dict[i]['g_id']
            
            output_dict.setdefault(GO_id, []).append(g_id)

    except:
        pass

    os.remove(GO_list_file)
    os.remove(foreground_gene_file)
    os.remove(background_GO_file)
    os.remove(output_file)

    return output_dict    

if __name__ == "__main__":

    """
    GO:0005575  cellular component
    GO:0003674  molecular function
    GO:0008150  biological process
    """


    obo_file = "/lustre/home/xuyuxing/Database/GO/go.obo"

    # load file
    ontology = Obo(obo_file)

    # get GO info
    ontology.get("GO:0000002")

    # get children
    ontology.find_children("GO:0000002")

    # get all children
    ontology.find_children("GO:0008150", expand=True)

    # get parents
    ontology.find_parents("GO:0000002")

    # get all parents
    ontology.find_parents("GO:0000012", expand=True)

    # get all path
    list(ontology.get_all_path("GO:0000012"))

    # %%
    # for wangming
    go_id_file = '/lustre/home/xuyuxing/Work/Other/wangming/go.id'

    from lib.common.fileIO import read_list_file
    from lib.common.util import printer_list

    go_id_list = read_list_file(go_id_file)
    # go_id_list = go_id_list[0:5]

    obo_file = "/lustre/home/xuyuxing/Database/GO/go.obo"

    # load file
    ontology = Obo(obo_file)

    with open("/lustre/home/xuyuxing/Work/Other/wangming/go.id.depth", 'w') as f:
        num = 0
        for i in go_id_list:
            num += 1
            print(num)
            try:
                path_dict = ontology.get_all_path(i)
                for root_id in path_dict:
                    depth_list = []
                    if len(path_dict[root_id]) == 0:
                        continue
                    for j in path_dict[root_id]:
                        depth_list.append(len(j) - 1)
                    depth_list = list(set(depth_list))
                    print(i,root_id,depth_list)
                    f.write("%s\t%s\t%s\n" % (i,root_id,printer_list(depth_list,',')))
            except:
                pass
        





from math import ceil
import re


# windows

def cover_windows_index(pos, step, window, start=1):
    """
    For a known windows set for a sequence (given step length, windows length, and start site).
    Now when I am on a position(pos), which windows covered me ?=
    :param pos: Pos site I am
    :param step: step for windows
    :param window: window length for windows
    :param start: where is start for first window
    :return: windows index (begin for 0) for windows which covered me
    """
    if pos < start:
        raise ValueError("Pos: %d is small than start %d" % (pos, start))
    else:
        pos = pos - start + 1
        for i in range(ceil((pos - window) / step), int((pos - 1) / step) + 1):
            if i >= 0:
                yield i


def windows_index_to_range(index, step, window, start=1):
    """
    For a known windows set for a sequence (given step length, windows length, and start site).
    Now I give a index (begin for 0), give me the detail for this window
    :param pos: index I have
    :param step: step for windows
    :param window: window length for windows
    :param start: where is start for first window
    :return: windows start and end
    """
    start = index * step + start
    end = start + window - 1
    return start, end


def total_windows(end, step, window, start=1):
    """
    return how many windows can be get for a given sequence and given windows parameter
    :param end: sequence length or end for window split
    :param step: step for windows
    :param window: windows length
    :param start: where is the start for the first window, default: 1
    :return: windows number
    """

    # seq_len = end - start + 1
    seq_len = end - start
    win_num = ceil((seq_len - window + 1) / step) + 1
    return win_num


def split_sequence_to_windows(end, step, window, start=1):
    """
    make a windows split for a sequence
    :param end: sequence length or end for window split
    :param step: step for windows
    :param window: windows length
    :param start: where is the start for the first window, default 1
    :return: generator for windows index, start and end
    """

    for i in range(0, total_windows(end, step, window, start)):
        start_tmp, end_tmp = windows_index_to_range(i, step, window, start)
        start_tmp, end_tmp = round(start_tmp), round(end_tmp)
        if start_tmp == end:
            yield i, start_tmp, end
            continue
        elif start_tmp > end:
            continue
        #     raise ValueError("start_tmp %d > end %d, it should not happen" % (start_tmp, end))
        elif end_tmp > end:
            yield i, start_tmp, end
        else:
            yield i, start_tmp, end_tmp


# bin (means windows without overlap)

def cover_bin_index(pos, bin_length, start=1):
    """
    For a known bins set for a sequence (given bin length and start site).
    Now when I am on a position(pos), which bin covered me ?
    :param pos: Pos site I am
    :param bin_length: length for a bin
    :param start: where is the start for the first bin
    :return: bin index (begin for 0) for windows which covered me
    """
    return list(cover_windows_index(pos, bin_length, bin_length, start))[0]


def bin_index_to_range(index, bin_length, start=1):
    """
    For a known bins set for a sequence (given bin_length, and start site).
    Now I give a index (begin for 0), give me the detail for this bin
    :param index: index I have
    :param bin_length: length of each bin
    :param start: where is start for first bin
    :return: bin start and end
    """
    return windows_index_to_range(index, bin_length, bin_length, start)


def total_bins(end, bin_length, start=1):
    """
    return how many bin can be get for a given bin_length
    :param end: sequence length or end for bin split
    :param bin_length: length of each bin
    :param start: where is the start for the first bin, default: 1
    :return: bins number
    """
    return total_windows(end, bin_length, bin_length, start)


def split_sequence_to_bins(end, bin_length, start=1):
    """
    make a windows split for a sequence
    :param end: sequence length or end for window split
    :param step: step for windows
    :param window: windows length
    :param start: where is the start for the first window, default 1
    :return: generator for windows index, start and end
    """
    return split_sequence_to_windows(end, bin_length, bin_length, start)



from api.xuyuxing.comp_genome.planttribes import seq_classify
from BCBio import GFF
from interlap import InterLap
from lib.common.evolution.orthotools2 import OrthoGroups, OrthoGroup
from lib.common.genome.genome_feature2 import Gene, HomoPredictResults
from lib.common.math.interval import overlap_between_interval_set, group_by_intervals_with_overlap_threshold, sum_interval_length
from lib.common.os import mkdir, multiprocess_running
from lib.common.util import logging_init, configure_parser
from src.xuyuxing.GenomeTools.WPGmapper import load_map_files, extract_all_evidences
import configparser
import os
import pickle


def mark_map_quality_for_gf_dict(input_gf_dict, args):
    for ev_id in input_gf_dict:
        # quality control
        coverage_flag = input_gf_dict[ev_id].evidence_indicator["query_coverage"] >= args.min_cover
        aln_aa_len_flag = input_gf_dict[ev_id].evidence_indicator["aln_aa_len"] >= args.min_aa_len
        identity_flag = input_gf_dict[ev_id].evidence_indicator["identity"] >= args.min_identity
        score_flag = input_gf_dict[ev_id].evidence_indicator["score"] >= args.min_score
        # parent_evalue_flag = input_gf_dict[ev_id].parent_blast["evalue"] <= args.evalue
        # parent_blast_identity_flag = input_gf_dict[ev_id].parent_blast["identity"] >= args.min_identity

        # Special arrow
        # special_arrow_flag = input_gf_dict[ev_id].evidence_indicator["query_coverage"] >= 0.8 and input_gf_dict[ev_id].evidence_indicator["identity"] >= 0.9 and input_gf_dict[
        #     ev_id].parent_blast["identity"] >= 0.9 and input_gf_dict[ev_id].evidence_indicator["aln_aa_len"] >= 200 and input_gf_dict[ev_id].parent_blast["evalue"] <= 1e-30

        # if coverage_flag and aln_aa_len_flag and identity_flag and score_flag and parent_evalue_flag and parent_blast_identity_flag:
        if coverage_flag and aln_aa_len_flag and identity_flag and score_flag:
            input_gf_dict[ev_id].map_quality_pass = True
        # elif score_flag and special_arrow_flag:
        #     input_gf_dict[ev_id].map_quality_pass = True
        else:
            input_gf_dict[ev_id].map_quality_pass = False

    return input_gf_dict


def parse_conserved_species_string(conserved_species_string):
    conserved_arguments = [[], []]

    for group_string in conserved_species_string.split(";"):
        group_info = group_string.split(",")
        if len(group_info) == 1:
            conserved_arguments[0].append(group_info)
            conserved_arguments[1].append(1)
        else:
            conserved_arguments[0].append(group_info[:-1])
            min_num = int(group_info[-1])
            conserved_arguments[1].append(min_num)

    return conserved_arguments


def get_HPR_from_evidence_dict(query_gene_list, evidence_dict, subject_species=None):
    query_gene_dict = {i.id: i for i in query_gene_list}

    hit_gene_dir = {}
    for gf_id in evidence_dict:
        gf = evidence_dict[gf_id]
        query_id = gf.evidence_indicator['query_id']

        if not query_id in hit_gene_dir:
            hit_gene_dir[query_id] = []
        hit_gene_dir[query_id].append(gf)

    output_list = []
    for query_id in hit_gene_dir:
        query_gene = query_gene_dict[query_id]
        output_list.append(
            HomoPredictResults(query_gene, subject_species=subject_species, hit_gene_list=hit_gene_dir[query_id]))

    return output_list


def merge_patch_hits(patch_hits):
    # merge as site
    site_dict = {}
    for gf in patch_hits:
        if not gf.chr_id in site_dict:
            site_dict[gf.chr_id] = {
                "+": InterLap(),
                "-": InterLap(),
            }
        site_dict[gf.chr_id][gf.strand].add((gf.start, gf.end, gf))

    output_gf_list = []
    for i in site_dict:
        for j in site_dict[i]:
            merge_groups = group_by_intervals_with_overlap_threshold(
                {k[2]: (k[0], k[1]) for k in site_dict[i][j]}, 0.8)

            for gp_id in merge_groups:
                hit_list = merge_groups[gp_id]['list']
                model_hit = sorted(hit_list, key=lambda x: len(
                    x.model_aa_seq.seq), reverse=True)[0]
                output_gf_list.append(model_hit)

    return output_gf_list


def GeneLoss_main(args):
    mkdir(args.work_dir, True)

    logger = logging_init("GeneLoss", args.log_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    # step1
    logger.info("Step1: read orthogroups file")
    orthogroups_tsv_file = args.orthogroup_scaffold_dir + "/orthogroups.tsv"
    OGs = OrthoGroups(OG_tsv_file=orthogroups_tsv_file)
    conserved_arguments = parse_conserved_species_string(
        args.conserved_species)
    conserved_og_id_list = OGs.get_conserved_OG_list(conserved_arguments)
    csv_OGs = OrthoGroups(from_OG_list=[OGs.OG_dict[i]
                                        for i in conserved_og_id_list])
    logger.info("       there are %d species, %d orthogroups, %d are conserved" % (
        len(OGs.species_list), len(OGs.OG_id_list), len(csv_OGs.OG_id_list)))

    # step2
    logger.info("Step2: get candidate gene loss orthologues in %s" %
                args.target_speci)
    candi_loss_list = [og_id for og_id in csv_OGs.OG_dict if len(
        csv_OGs.OG_dict[og_id].gene_dict[args.target_speci]) == 0]
    logger.info("       there are %d (%.2f%%) orthogroups don't have gene in %s" % (len(
        candi_loss_list), len(candi_loss_list)/len(csv_OGs.OG_id_list)*100, args.target_speci))
    cand_loss_OGs = OrthoGroups(
        from_OG_list=[OGs.OG_dict[i] for i in candi_loss_list])
    cand_loss_OGs.write_OG_tsv_file(
        args.work_dir + "/1.candidate_gene_loss.txt")

    # step3
    logger.info("Step3: get ref gene mapping results from WPGmapper")
    genewise_dict_file = args.work_dir + "/2.genewise_dict.pyb"

    ref_sp_list = []
    for i in conserved_arguments[0]:
        for j in i:
            ref_sp_list.append(j)
    ref_sp_list = list(set(ref_sp_list))

    if not os.path.exists(genewise_dict_file):
        logger.info("Step3.1: load data from WPGmapper")

        ref_gene_list = []
        for og_id in candi_loss_list:
            og = csv_OGs.OG_dict[og_id]
            for sp_id in ref_sp_list:
                for gene in og.gene_dict[sp_id]:
                    ref_gene_list.append(gene)

        evidence_id_list = []
        for gene in ref_gene_list:
            evidence_id_list.extend([gene.id+"_"+str(i)
                                     for i in range(args.top_evidence_num)])

        ref_WPGmapper_dict = load_map_files(
            ref_sp_list, args.wpgmapper_dir)

        evidence_dict = extract_all_evidences(
            ref_WPGmapper_dict, evidence_id_list=evidence_id_list, log_file=args.log_file)

        evidence_dict = mark_map_quality_for_gf_dict(evidence_dict, args)

        logger.info("Step3.1: begin compare")
        # load genewise output
        genewise_out = get_HPR_from_evidence_dict(
            ref_gene_list, evidence_dict, subject_species=args.target_speci)

        # load annotated range
        annotated_range_dict = {}
        gene_cds_intervals_dict = {}
        with open(args.target_speci_annotation, 'r') as in_handle:
            for rec in GFF.parse(in_handle):
                if rec.id not in annotated_range_dict:
                    annotated_range_dict[rec.id] = {}
                    annotated_range_dict[rec.id]["+"] = InterLap()
                    annotated_range_dict[rec.id]["-"] = InterLap()
                for gene in rec.features:
                    if args.target_speci_feature == 'None' or gene.type == args.target_speci_feature:
                        start_tmp = gene.location.start.position + 1
                        end_tmp = gene.location.end.position
                        if gene.id == "":
                            gene.id = list(
                                set(gene.qualifiers.keys()) - {'source'})[0]
                        if gene.strand == 1:
                            annotated_range_dict[rec.id]["+"].add(
                                (start_tmp, end_tmp, gene.id))
                        else:
                            annotated_range_dict[rec.id]["-"].add(
                                (start_tmp, end_tmp, gene.id))

                        gene_cds_intervals_dict[gene.id] = [(int(i.location.start)+1, int(
                            i.location.end)) for i in gene.sub_features[0].sub_features if i.type == 'CDS']

        # mark new range which not in annotated range and have good coverage
        for query_HPR in genewise_out:
            for hit_gene in query_HPR.hit_gene_list:
                cds_range = [(i.start, i.end)
                             for i in hit_gene.sub_features[0].sub_features if i.type == 'CDS']
                cds_length = sum_interval_length(cds_range)

                mRNA_qual = hit_gene.sub_features[0].qualifiers

                # test if have been annotated
                loci_tmp = hit_gene.chr_loci
                hit_gene.new_anno = (True, None, 0.0)

                overlaped_gene_id = []

                if loci_tmp.chr_id in annotated_range_dict:
                    for annotated_range in annotated_range_dict[loci_tmp.chr_id][loci_tmp.strand].find(
                            (loci_tmp.start, loci_tmp.end)):
                        anno_s = annotated_range[0]
                        anno_e = annotated_range[1]
                        known_gene_id = annotated_range[2]
                        overlaped_gene_id.append(known_gene_id)

                overlaped_gene_id = list(set(overlaped_gene_id))

                for gene_id in overlaped_gene_id:
                    gene_cds_range = gene_cds_intervals_dict[gene_id]
                    overlap_ratio, overlap_length, overlap = overlap_between_interval_set(
                        cds_range, gene_cds_range)
                    overlap_ratio = overlap_length/cds_length

                    if overlap_ratio >= args.annotated_coverage:
                        hit_gene.new_anno = (
                            False, known_gene_id, overlap_ratio)
                    else:
                        hit_gene.new_anno = (
                            True, known_gene_id, overlap_ratio)

        # turn genewise_out to a dict, key is query_name
        genewise_dict = {}
        for query_HPR in genewise_out:
            genewise_dict[query_HPR.query_gene.id] = query_HPR

        OUT = open(genewise_dict_file, 'wb')
        pickle.dump(genewise_dict, OUT)
        OUT.close()
    else:
        logger.info("comparsion already finished")
        TEMP = open(genewise_dict_file, 'rb')
        genewise_dict = pickle.load(TEMP)
        TEMP.close()

    logger.info(
        "%d query seq get %d genewise hit, %d can pass coverage test, %d are on the known gene loci, %d are good new hit" % (
            len(genewise_dict),
            sum([len(genewise_dict[i].hit_gene_list) for i in genewise_dict]),
            sum([len([j for j in genewise_dict[i].hit_gene_list if j.map_quality_pass]) for i in
                 genewise_dict]),
            sum([len([j for j in genewise_dict[i].hit_gene_list if j.map_quality_pass and not j.new_anno[0]])
                 for i in genewise_dict]),
            sum([len(
                [j for j in genewise_dict[i].hit_gene_list if j.map_quality_pass and j.new_anno[0]])
                for i in genewise_dict])
        ))

    # step4 use seq_classify to check if new hit's gene family
    logger.info("Step4: use seq_classify to check if new hit's gene family")

    seq_classify_dict_file = args.work_dir + "/3.seq_classify_dict.pyb"

    if not os.path.exists(seq_classify_dict_file):
        seq_classify_dir = args.work_dir + "/seq_classify"
        mkdir(seq_classify_dir, True)

        good_hit_list = []
        for og_id in candi_loss_list:
            og = csv_OGs.OG_dict[og_id]
            for sp_id in ref_sp_list:
                for gene in og.gene_dict[sp_id]:
                    if gene.id in genewise_dict:
                        for hit in genewise_dict[gene.id].hit_gene_list:
                            if hit.new_anno[0]:
                                good_hit_list.append(hit)

        args_list = []
        args_id_list = []
        for hit in good_hit_list:
            sub_work_dir = seq_classify_dir + "/" + hit.id
            args_list.append(
                (hit.model_aa_seq, args.orthogroup_scaffold_dir, sub_work_dir, True))
            args_id_list.append(hit.id)

        mlt_out = multiprocess_running(
            seq_classify, args_list, args.num_threads, args_id_list=args_id_list, log_file=args.log_file)

        seq_classify_dict = {}
        for i in mlt_out:
            seq_classify_dict[i] = mlt_out[i]['output']
            # seq_classify_dict[mlt_out[i]['args'][2].split("/")[-1]] = mlt_out[i]['output']

        OUT = open(seq_classify_dict_file, 'wb')
        pickle.dump(seq_classify_dict, OUT)
        OUT.close()

    else:
        TEMP = open(seq_classify_dict_file, 'rb')
        seq_classify_dict = pickle.load(TEMP)
        TEMP.close()

    # step5 decide if orthogroups lost in target
    logger.info("Step5: decide if orthogroups lost in target")

    reanno_og_dict = {}
    for og_id in candi_loss_list:
        og = csv_OGs.OG_dict[og_id]

        patch_hits = []
        for sp_id in ref_sp_list:
            for gene in og.gene_dict[sp_id]:
                if gene.id in genewise_dict:
                    for hit in genewise_dict[gene.id].hit_gene_list:
                        if hit.new_anno[0]:
                            if hit.id in seq_classify_dict:
                                if seq_classify_dict[hit.id] == og_id:
                                    patch_hits.append(hit)

        if len(patch_hits) > 0:
            reanno_og_dict[og_id] = merge_patch_hits(patch_hits)

    # step6 output
    logger.info("Step6: make output")

    new_csv_og_list = []
    for og_id in csv_OGs.OG_dict:
        og = csv_OGs.OG_dict[og_id]

        if og_id in reanno_og_dict:
            gene_list = og.gene_list
            for hit in reanno_og_dict[og_id]:
                gene_list.append(Gene(id=hit.id, species=args.target_speci))

            og = OrthoGroup(id=og.id, from_gene_list=gene_list,
                            species_list=og.species_list)

        new_csv_og_list.append(og)

    new_csv_OGs = OrthoGroups(from_OG_list=new_csv_og_list)
    new_csv_OGs.write_OG_tsv_file(args.work_dir + "/4.gene_loss.tsv")

    logger.info("In %d conserved orthogroups, %d (%.2f%%) orthogroups don't have gene in %s, %d can find back, %d (%.2f%%) are truly lost" % (len(csv_OGs.OG_id_list), len(candi_loss_list), len(
        candi_loss_list)/len(csv_OGs.OG_id_list)*100, args.target_speci, len(reanno_og_dict), len(candi_loss_list) - len(reanno_og_dict), (len(candi_loss_list) - len(reanno_og_dict))/len(csv_OGs.OG_id_list)*100))


def GeneLoss_args_parser(args):

    # configure file argument parse

    try:
        script_dir_path = os.path.split(os.path.realpath(__file__))[0]
        defaults_config_file = script_dir_path + \
            "/GeneLoss2_defaults.ini"
    except:
        defaults_config_file = "/lustre/home/xuyuxing/python_project/Genome_work_tools/src/xuyuxing/GenomeTools/GeneLoss2_defaults.ini"
    # print(defaults_config_file)

    args_type = {
        # str
        "config_file": "str",
        "conserved_species": "str",
        "orthogroup_scaffold_dir": "str",
        "wpgmapper_dir": "str",
        "target_speci": "str",
        "target_speci_annotation": "str",
        "work_dir": "str",
        "log_file": "str",
        "target_speci_feature": "str",

        # float
        "min_cover": "float",
        "min_identity": "float",
        "annotated_coverage": "float",

        # int
        "num_threads": "int",
        "top_evidence_num": "int",
        "min_aa_len": "int",
        "min_score": "int",
    }

    args = configure_parser(args, defaults_config_file,
                            args.config_file, args_type, None)

    # Modify_dict parse
    cfg = configparser.ConfigParser()
    cfg.read(args.config_file)

    # geneloss running

    if args.log_file is None:
        args.log_file = args.work_dir + "/log"

    GeneLoss_main(args)


if __name__ == '__main__':
    # command argument parse
    class abc(object):
        pass

    args = abc()

    args.conserved_species = "Ath;Ini;Llu;Cca,Sly,Oeu,Sin,Mgu,3"
    args.orthogroup_scaffold_dir = "/lustre/home/xuyuxing/Work/Orobanchaceae/gene_loss/orthologous_scaffold"
    args.wpgmapper_dir = "/lustre/home/xuyuxing/Work/Orobanchaceae/gene_loss/WPGmapper/Ocu_WPGmapper"
    args.target_speci = "Ocu"
    args.target_speci_annotation = "/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Orobanche_cumana/T78542N0.genome.gff3"
    args.work_dir = "/lustre/home/xuyuxing/Work/Orobanchaceae/gene_loss/gene_loss/Ocu"
    args.log_file = "/lustre/home/xuyuxing/Work/Orobanchaceae/gene_loss/gene_loss/Ocu/log"
    args.num_threads = 56

    args.top_evidence_num = 20
    args.min_cover = 0.5
    args.min_aa_len = 50
    args.min_identity = 0.3
    args.min_score = 50
    args.target_speci_feature = 'gene'
    args.annotated_coverage = 0.8

    GeneLoss_main(args)



__author__ = 'Yuxing Xu'


if __name__ == '__main__':
    import argparse

    # argument parse
    parser = argparse.ArgumentParser(
        prog='TransTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for TrinityGene
    parser_a = subparsers.add_parser('TrinityGene',
                                     help='Extract longest isoform from a gene in trinity output\n',
                                     description='Extract longest isoform from a gene in trinity output')

    parser_a.add_argument('Trinity_output', type=str,
                          help='a fasta file from trinity output')
    parser_a.add_argument('Trinity_gene', type=str,
                          help='a fasta file from this script')

    # argparse for GenerateTrinityGeneTransMap
    parser_a = subparsers.add_parser('GenerateTrinityGeneTransMap',
                                     help='Generate Trinity Gene Trans Map file from trinity fasta\n'
                                     )

    parser_a.add_argument('Trinity_output', type=str,
                          help='a fasta file from trinity output')
    parser_a.add_argument('gene_map_file', type=str,
                          help='a tab file for gene_map')

    # argparse for Denovo2RefCountMap
    parser_a = subparsers.add_parser('Denovo2RefCountMap',
                                     help='Use denovo trinity make gene count, and map it to a genome\n'
                                     )

    parser_a.add_argument('gene_model_cds_file', type=str,
                          help='cds file from reference genome')
    parser_a.add_argument('tran_fasta_file', type=str,
                          help='a fasta file from trinity output')
    parser_a.add_argument('gene_trans_map', type=str,
                          help='a gene_trans_map file from trinity output')
    parser_a.add_argument('-f', '--tran_count_fof', type=str, default=None,
                          help='a list of rsem genes.results file: (sampleid rsem_count_file)')
    parser_a.add_argument('-c', '--tran_count_matrix', type=str, default=None,
                          help='a count matrix')
    parser_a.add_argument('-w', '--work_dir', type=str, default='tmp',
                          help='tmp work dir')
    parser_a.add_argument('-o', '--output_prefix', type=str, default='Gene',
                          help='output prefix')
    parser_a.add_argument('-t', '--threads', type=int, default=56,
                          help='number of threads: defaults 56')

    # argparse for Count2TMM
    parser_a = subparsers.add_parser('Count2TMM',
                                     help='from count matrix 2 TMM matrix, FPKM and TPM will give to\n'
                                     )

    parser_a.add_argument('count_matrix', type=str,
                          help='a count matrix')
    parser_a.add_argument('gene_length_file', type=str,
                          help='gene length file')
    parser_a.add_argument('output_prefix', type=str,
                          help='output prefix')


    # argparse for ContaminationDetector
    parser_a = subparsers.add_parser('ContaminationDetector',
                                     help='find contamination from diamond output: diamond blastp --query Trinity.model.faa --max-target-seqs 10 --db /lustre/home/xuyuxing/Database/NCBI/nr/2020/nr.taxon.dmnd --evalue 1e-5 --out Trinity.model.faa.bls --outfmt 6 qseqid sseqid staxids pident length mismatch gapopen qstart qend sstart send evalue bitscore --threads 56\n'
                                     )

    parser_a.add_argument('bls_results_file', type=str,
                          help='diamond blast output')
    parser_a.add_argument('taxon_db_file', type=str,
                          help='taxon_db_file, from ncbi taxonomy parsed by TaxonTools')
    parser_a.add_argument('target_taxon', type=str,
                          help='can be taxon sciname or taxon id')

    args = parser.parse_args()
    args_dict = vars(args)

    # --------------------------------------------
    # command detail

    # TrinityGene
    if args_dict["subcommand_name"] == "TrinityGene":
        from src.xuyuxing.tools.seqtools import TrinityGene_main
        TrinityGene_main(args)

    # GenerateTrinityGeneTransMap
    elif args_dict["subcommand_name"] == "GenerateTrinityGeneTransMap":
        from src.xuyuxing.tools.seqtools import GenerateTrinityGeneTransMap_main
        GenerateTrinityGeneTransMap_main(args)

    # Denovo2RefCountMap
    elif args_dict["subcommand_name"] == "Denovo2RefCountMap":
        from src.xuyuxing.Transcriptome.Denovo2RefCountMap import Denovo2RefCountMap_main
        Denovo2RefCountMap_main(args)

    # Count2TMM
    elif args_dict["subcommand_name"] == "Count2TMM":
        from src.xuyuxing.Transcriptome.Denovo2RefCountMap import Count2TMM_main
        Count2TMM_main(args)

    # ContaminationDetector
    elif args_dict["subcommand_name"] == "ContaminationDetector":
        from src.xuyuxing.Transcriptome.contamination import detect_contaminate_from_diamond
        detect_contaminate_from_diamond(args.bls_results_file, args.taxon_db_file, args.target_taxon)



from lib.common.genome.seq_base import read_fasta_by_faidx
from api.common.genome.blast import outfmt6_read_big
import lib.common.sqlite_command as sc
from lib.common.math.bin_and_window import split_sequence_to_bins
import pickle
import re

close_group_bls = "/lustre/home/xuyuxing/Work/Gel/HGT/nt_genome_HGT/plant_bls/plant.bls"
distant_group_bls = "/lustre/home/xuyuxing/Work/Gel/HGT/nt_genome_HGT/fungi_bls/simple_split/fungi.bls"
genome_fasta = "/lustre/home/xuyuxing/Work/Gel/GenomeStat/T91201N0.genome.fasta"
tmp_dir = "/lustre/home/xuyuxing/Work/Gel/GenomeStat/HGT"
bin_length=100

def simple_nt_HGT_stat(genome_fasta, close_group_bls, distant_group_bls, bin_length=100, tmp_dir="/tmp"):
    """
    DOI: 10.1016/j.cub.2020.12.045
    from Figure 5C
    """
    def parse_function(blast_results):
        return {'record': (blast_results.qID, sc.pickle_dump_obj(blast_results))} 

    def extract_function(query_id_list, bls_db):
        return {i[0]:sc.pickle_load_obj(i[1]) for i in sc.sqlite_select(bls_db, 'record', key_name='id', value_tuple=query_id_list)}

    def get_similarity(bls_record, bin_length=None, use_identity=True):
        if use_identity:
            similarity = bls_record.hit[0].hsp[0].Hsp_identity / 100
            return similarity
        else:
            # similarity = min(1.0, bls_record.hit[0].hsp[0].Hsp_identity / 100 * bls_record.hit[0].hsp[0].Hsp_align_len / bin_length)
            similarity = min(1.0, bls_record.hit[0].hsp[0].Hsp_align_len / bin_length)
            return similarity

    # load blast into db
    distant_bls_db = tmp_dir + "/distant_bls.db"
    sc.build_database(outfmt6_read_big(distant_group_bls), parse_function, {'record':['id', 'codenstring']}, distant_bls_db)
    sc.build_index(distant_bls_db, 'record', 'id')

    close_bls_db = tmp_dir + "/close_bls.db"
    sc.build_database(outfmt6_read_big(close_group_bls), parse_function, {'record':['id', 'codenstring']}, close_bls_db)
    sc.build_index(close_bls_db, 'record', 'id')

    # bin HGT
    chr_dict = read_fasta_by_faidx(genome_fasta)

    key_list = []
    for chr_id in chr_dict:
        chr_length = chr_dict[chr_id].len()
        for i, s, e in split_sequence_to_bins(chr_length, bin_length, 1):
            key_str = "%s_%d-%d" % (chr_id, s, e)
            key_list.append(key_str)
    
    step = 10000
    sim_dict = {}
    for i in range(0, len(key_list), step):
        # print(i)
        sub_key_list = key_list[i:i+step]
        close_record = extract_function(sub_key_list, close_bls_db)
        distant_record = extract_function(sub_key_list, distant_bls_db)

        # get similarity
        for k in sub_key_list:
            if k in close_record:
                cr = close_record[k]
            else:
                cr = None

            if k in distant_record:
                dr = distant_record[k]
            else:
                dr = None

            if cr:
                crs = get_similarity(cr, bin_length=bin_length, use_identity=False)
                drs = 0.0
            elif dr:
                crs = 0.0
                drs = get_similarity(dr, bin_length=bin_length, use_identity=False)
            else:
                crs=0.0
                drs=0.0

            sim_dict[k] = (crs,drs,cr,dr)

    output_dict = {}
    for k in sim_dict:
        chr_id, s, e = re.findall(r'(\S+)_(\S+)-(\S+)', k)[0]
        if chr_id not in output_dict:
            output_dict[chr_id] = {}
        output_dict[chr_id][(int(s),int(e))] = sim_dict[k]

    return output_dict



import networkx as nx
import matplotlib.pyplot as plt
import numpy as np


def plot_graph(G, pos, figsize=(20, 20), save_file=None, node_color_dict=None, node_size_dict=None, node_label_list=None):
    fig, ax = plt.subplots(figsize=figsize)

    if node_color_dict:
        node_color = [node_color_dict[node_id] for node_id in pos]
    else:
        node_color = '#1f78b4'

    if node_size_dict:
        node_size = [node_size_dict[node_id] for node_id in pos]
    else:
        node_size = 600

    nx.draw_networkx_nodes(G, pos, edgecolors='k', linewidths=1,
                           node_size=node_size, node_color=node_color)

    if node_label_list:
        labels_dict = dict((n, n)
                           for n in G.nodes() if n in set(node_label_list))
        nx.draw_networkx_labels(
            G, pos, labels=labels_dict, font_size=10, font_weight='bold', font_color='k')
    else:
        nx.draw_networkx_labels(G, pos, font_size=10,
                                font_weight='bold', font_color='k')
    nx.draw_networkx_edges(G, pos, edge_color="#8D8D8D", width=0.5)

    # nx.draw_networkx(G, ax=ax)
    print(len(G.nodes), len(G.edges))

    plt.show()

    if save_file:
        fig.savefig(save_file, format='pdf', facecolor='none',
                    edgecolor='none', bbox_inches='tight')


def remove_overmuch_degree(G, max_degree):
    all_overmuch_edge_list = []
    for node in G.nodes:
        nb_list = list(G.neighbors(node))
        sorted_nb_list = sorted(nb_list, key=lambda x: G.get_edge_data(x, node)[
                                'weight'], reverse=True)
        if len(sorted_nb_list) > max_degree:
            overmuch_edge_list = [tuple(sorted([node, nb]))
                                  for nb in sorted_nb_list[max_degree:]]
            all_overmuch_edge_list.extend(overmuch_edge_list)

    G.remove_edges_from(all_overmuch_edge_list)

    return G


def remove_low_weight_edges(G, min_weight):
    low_weight_edge_list = [e for e in G.edges if G.get_edge_data(e[0], e[1])[
        'weight'] < min_weight]
    G.remove_edges_from(low_weight_edge_list)

    return G


def remove_low_degree_node(G, min_degree):

    low_degree_nodes = []
    for node in G.nodes:
        if G.degree(node) <= min_degree:
            low_degree_nodes.append(node)

    G.remove_nodes_from(low_degree_nodes)

    return G


def get_subGraph(G):
    sub_Gs = [G.subgraph(c) for c in nx.connected_components(G)]
    return sub_Gs


def get_plot_pos(G):
    pos = nx.drawing.spring_layout(G)
    return pos


if __name__ == '__main__':

    G = nx.Graph()
    G.add_nodes_from(node_list)
    G.add_weighted_edges_from(edge_list)




import re
from lib.common.math.interval import sum_interval_length


class repeat_case(object):
    def __init__(self, SW_score, perc_div, perc_del, perc_ins, query, begin, end, left, strand, repeat_id, repeat_class,
                 repeat_range1, repeat_range2, repeat_range3, ID=None, star=False):
        self.SW_score = int(SW_score)
        self.perc_div = float(perc_div)
        self.perc_del = float(perc_del)
        self.perc_ins = float(perc_ins)
        self.q_name = query
        self.q_start = int(begin)
        self.q_end = int(end)

        # q_length
        left = int(re.search(r'^\((.*)\)$', left).groups()[0])
        self.q_length = self.q_end + left

        # strand
        if strand == "C":
            self.strand = "-"
        else:
            self.strand = "+"

        self.repeat_name = repeat_id
        self.repeat_class = repeat_class

        # repeat range
        if self.strand == "+":
            self.r_start = int(repeat_range1)
            self.r_end = int(repeat_range2)
            repeat_range3 = int(re.search(r'^\((.*)\)$', repeat_range3).groups()[0])
            self.r_length = int(repeat_range2) + repeat_range3
        else:
            self.r_start = int(repeat_range3)
            self.r_end = int(repeat_range2)
            repeat_range1 = int(re.search(r'^\((.*)\)$', repeat_range1).groups()[0])
            self.r_length = int(repeat_range2) + repeat_range1

        if ID:
            self.ID = ID
        else:
            self.ID = None


        if star:
            self.star = True
        else:
            self.star = False

    def __str__(self):
        return "ID: %s\n" \
               "query: %s\n" \
               "begin: %d\n" \
               "end: %d\n" \
               "strand: %s\n" \
               "repeat_class: %s\n" \
               "repeat_id: %s\n" \
               "repeat_begin: %d\n" \
               "repeat_end: %d\n" \
               "repeat_length: %d\n" \
               "repeat_coverage: %.2f\n" \
               "div: %.1f\n" \
               "del: %.1f\n" \
               "ins: %.1f\n" \
               "SW_score: %d\n" \
               "star: %s\n" % (
                   self.ID, self.q_name, self.q_start, self.q_end, self.strand, self.repeat_class, self.repeat_name,
                   self.r_start, self.r_end, self.r_length, (self.r_end - self.r_start) / self.r_length * 100,
                   self.perc_div,
                   self.perc_del, self.perc_ins, self.SW_score, self.star)


class repeat_family(object):
    def __str__(self):
        case_num = len(self.case_list)
        total_length = self.length_sum()
        return "repeat_name: %s\n" \
               "repeat_class: %s\n" \
               "case_num: %d\n" \
               "length_sum: %d\n" % (self.repeat_name, self.repeat_class, case_num, total_length)

    def __init__(self, repeat_name, repeat_class, case_list):
        self.repeat_name = repeat_name
        self.repeat_class = repeat_class
        self.case_list = case_list

    def length_sum(self):

        range_dict = {}
        for case_tmp in self.case_list:
            if case_tmp.q_name not in range_dict:
                range_dict[case_tmp.q_name] = []
            range_dict[case_tmp.q_name].append((case_tmp.q_start,case_tmp.q_end))

        total_length = 0
        for query in range_dict:
            total_length = total_length + sum_interval_length(range_dict[query])

        return total_length

def repeatmasker_parser(repeatmasker_outfile):
    """
    repeatmasker_outfile = "/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/repeat/denovo/Cuscuta.genome.v1.1.fasta.finial.out/Cuscuta.genome.v1.1.fasta.out"
    :param repeatmasker_out:
    :return:
    """

    repeat_category = ['SINE',
                       'DNA',
                       'DNAauto',
                       'scRNA',
                       'LINE',
                       'snRNA',
                       'RC',
                       'LTR',
                       'RNA',
                       'Retroposon',
                       'Low_complexity',
                       'Simple_repeat',
                       'rRNA',
                       'tRNA',
                       'Satellite',
                       'Unknown',
                       'Segmental',
                       'Unspecified']

    output_dir = {}
    with open(repeatmasker_outfile, "r") as f:
        num = 0
        for each_line in f.readlines():
            num = num + 1
            if num <= 3:
                continue
            each_line = each_line.strip()
            each_line = re.sub(r'\s+', ' ', each_line)
            info = each_line.split(" ")

            repeat_record = repeat_case(*info)

            head_class = repeat_record.repeat_class.split("/")[0]
            head_class = re.sub('\?', '', head_class )

            if head_class in repeat_category:
                if head_class not in output_dir:
                    output_dir[head_class] = {}
                repeat_family_name = repeat_record.repeat_name
                if repeat_family_name not in output_dir[head_class]:
                    output_dir[head_class][repeat_family_name] = repeat_family(repeat_family_name,head_class,[])
                output_dir[head_class][repeat_family_name].case_list.append(repeat_record)
            else:
                raise ValueError("category incomp: %s" % head_class)

    return output_dir


from lib.common.genome.genome_feature2 import Gene, read_gff_file
from lib.common.genome.seq_base import read_fasta_by_faidx, write_fasta
from lib.xuyuxing.base.common_command import merge_dict
from lib.common.util import printer_list
from lib.common.os import mkdir, cmd_run
from lib.common.fileIO import tsv_file_dict_parse
import re
from api.xuyuxing.comp_genome.orthofinder import OG_tsv_file_parse
from interlap import InterLap
from lib.common.math.set import merge_same_element_set
from lib.common.math.interval import section, merge_intervals
from collections import OrderedDict
from api.xuyuxing.comp_genome.orthofinder import tsv_to_gene_pair
import numpy as np
from api.common.genome.blast import outfmt6_read_big
from api.xuyuxing.plot.geneplot import add_box
import matplotlib.pyplot as plt

# running mcscanx


def get_chr_rename_map(chr_list, prefix):
    num = 1
    rename_map = {}
    rename_map_hash = {}
    for i in chr_list:
        num += 1
        rename_map[i] = prefix + str(num)
        rename_map_hash[prefix + str(num)] = i

    return rename_map, rename_map_hash


def write_mcscanx_gff(gf_dict, chr_rename_map, output_gff):
    with open(output_gff, 'w') as f:
        for gf_id in gf_dict:
            gf = gf_dict[gf_id]
            f.write("%s\t%s\t%d\t%d\n" %
                    (chr_rename_map[gf.chr_id], gf.id, gf.start, gf.end))


def blast_c_score_filter(blast_file, output_blast_file, c_score_threshold=0.5):
    """
    c_score=0.5 indicating their BLASTP bit-scores were below 50% of the bit-scores of the best matches
    only outfmt6
    """
    with open(output_blast_file, 'w') as f:
        for q_record in outfmt6_read_big(blast_file):
            best_bit_score = q_record.hit[0].hsp[0].Hsp_bit_score
            keep_hit = []
            for hit_tmp in q_record.hit:
                now_bit_score = hit_tmp.hsp[0].Hsp_bit_score
                c_score = now_bit_score / best_bit_score

                if c_score >= c_score_threshold:
                    keep_hit.append(hit_tmp)

            for hit_tmp in keep_hit:
                q_id = hit_tmp.query.qDef
                s_id = hit_tmp.Hit_def
                for hsp_tmp in hit_tmp.hsp:
                    identity = hsp_tmp.Hsp_identity
                    aln_len = hsp_tmp.Hsp_align_len
                    mis = hsp_tmp.Hsp_mismatch
                    gap = hsp_tmp.Hsp_gaps
                    q_start = hsp_tmp.Hsp_query_from
                    q_end = hsp_tmp.Hsp_query_to
                    s_start = hsp_tmp.Hsp_hit_from
                    s_end = hsp_tmp.Hsp_hit_to
                    e_value = hsp_tmp.Hsp_evalue
                    score = hsp_tmp.Hsp_bit_score

                prt = printer_list([q_id, s_id, identity, aln_len, mis,
                                    gap, q_start, q_end, s_start, s_end, e_value, score])
                f.write(prt+"\n")


def running_mcscanx(species1_gff3, species2_gff3, work_dir, species1_prefix='aa', species1_aa_fasta=None, species2_prefix='bb', species2_aa_fasta=None, skip_gene_list=[], give_bls_file=None, mcscanx_options='-k 50 -g -1 -s 5 -e 1e-05 -m 25 -a', c_score_threshold=0.5):
    # prepare gff file
    mkdir(work_dir, True)

    sp1_gff_dict = read_gff_file(species1_gff3)
    sp1_gf_dict = {}
    for gf_type in sp1_gff_dict:
        for i in sp1_gff_dict[gf_type]:
            if i not in skip_gene_list:
                sp1_gf_dict[i] = sp1_gff_dict[gf_type][i]

    sp2_gf_dict = {}
    if species2_gff3:
        sp2_gff_dict = read_gff_file(species2_gff3)
        for gf_type in sp2_gff_dict:
            for i in sp2_gff_dict[gf_type]:
                if i not in skip_gene_list:
                    sp2_gf_dict[i] = sp2_gff_dict[gf_type][i]

    sp1_chr_list = list(
        set([sp1_gf_dict[gf_id].chr_id for gf_id in sp1_gf_dict]))
    sp1_chr_map, sp1_chr_map_hash = get_chr_rename_map(
        sp1_chr_list, species1_prefix)

    sp2_chr_list = list(
        set([sp2_gf_dict[gf_id].chr_id for gf_id in sp2_gf_dict]))
    sp2_chr_map, sp2_chr_map_hash = get_chr_rename_map(
        sp2_chr_list, species2_prefix)

    merge_chr_map = merge_dict([sp1_chr_map, sp2_chr_map], extend_value=False)
    merge_gf_dict = merge_dict([sp1_gf_dict, sp2_gf_dict], extend_value=False)

    mcscanx_gff = work_dir + "/mcscanx.gff"
    write_mcscanx_gff(merge_gf_dict, merge_chr_map, mcscanx_gff)

    # prepare fasta file
    if give_bls_file is not None:
        bls_flag = True
    elif species1_aa_fasta is None:
        raise ValueError("Must have fasta or bls results!")
    else:
        bls_flag = False

    bls_raw_file = work_dir + "/raw_blast.bls"
    bls_file = work_dir + "/mcscanx.blast"

    if bls_flag:
        cmd_string = "cp %s %s" % (give_bls_file, bls_file)
        cmd_run(cmd_string)
    else:
        fasta_file = work_dir + "/mcscanx.fasta"
        sp1_aa_dict = read_fasta_by_faidx(species1_aa_fasta)
        sp2_aa_dict = {}
        if species2_aa_fasta:
            sp2_aa_dict = read_fasta_by_faidx(species2_aa_fasta)
        aa_dict = merge_dict([sp1_aa_dict, sp2_aa_dict], False)
        aa_list = [aa_dict[i] for i in aa_dict if i not in skip_gene_list]
        write_fasta(aa_list, fasta_file)

        cmd_string = "makeblastdb -in %s -dbtype prot" % fasta_file
        cmd_run(cmd_string)

        cmd_string = "blastp -query %s -db %s -out %s -outfmt 6 -evalue 1e-10 -num_threads 56" % (
            fasta_file, fasta_file, bls_raw_file)
        cmd_run(cmd_string)

        blast_c_score_filter(bls_raw_file, bls_file,
                             c_score_threshold=c_score_threshold)

    # run mcscanx
    cmd_string = "MCScanX mcscanx " + mcscanx_options
    cmd_run(cmd_string, cwd=work_dir)

    # write map file
    sp1_map_file = work_dir + "/" + species1_prefix + ".map"

    with open(sp1_map_file, 'w') as f:
        for i in sp1_chr_map:
            f.write("%s\t%s\n" % (i, sp1_chr_map[i]))

    if species2_gff3:
        sp2_map_file = work_dir + "/" + species2_prefix + ".map"

        with open(sp2_map_file, 'w') as f:
            for i in sp2_chr_map:
                f.write("%s\t%s\n" % (i, sp2_chr_map[i]))


def running_mcscanxh(species1_gff3, species2_gff3, OG_tsv_file, work_dir, species1_prefix='aa', species2_prefix='bb', mcscanx_options='-k 50 -g -1 -s 5 -e 1e-05 -m 25 -a', huge_gene_family_filter=None):
    # prepare gff file
    mkdir(work_dir, True)

    sp1_gff_dict = read_gff_file(species1_gff3)
    sp1_gf_dict = {}
    for gf_type in sp1_gff_dict:
        for i in sp1_gff_dict[gf_type]:
            sp1_gf_dict[i] = sp1_gff_dict[gf_type][i]

    sp2_gf_dict = {}
    if species2_gff3:
        sp2_gff_dict = read_gff_file(species2_gff3)
        for gf_type in sp2_gff_dict:
            for i in sp2_gff_dict[gf_type]:
                sp2_gf_dict[i] = sp2_gff_dict[gf_type][i]

    sp1_chr_list = list(
        set([sp1_gf_dict[gf_id].chr_id for gf_id in sp1_gf_dict]))
    sp1_chr_map, sp1_chr_map_hash = get_chr_rename_map(
        sp1_chr_list, species1_prefix)

    sp2_chr_list = list(
        set([sp2_gf_dict[gf_id].chr_id for gf_id in sp2_gf_dict]))
    sp2_chr_map, sp2_chr_map_hash = get_chr_rename_map(
        sp2_chr_list, species2_prefix)

    merge_chr_map = merge_dict([sp1_chr_map, sp2_chr_map], extend_value=False)
    merge_gf_dict = merge_dict([sp1_gf_dict, sp2_gf_dict], extend_value=False)

    mcscanx_gff = work_dir + "/mcscanx.gff"
    write_mcscanx_gff(merge_gf_dict, merge_chr_map, mcscanx_gff)

    # prepare homology
    tsv_to_gene_pair(OG_tsv_file, species1_prefix,
                     species2_prefix, work_dir + "/mcscanx.homology", huge_gene_family_filter=huge_gene_family_filter)

    # run mcscanxh
    cmd_string = "MCScanX_h mcscanx " + mcscanx_options
    cmd_run(cmd_string, cwd=work_dir)

    # write map file
    sp1_map_file = work_dir + "/" + species1_prefix + ".map"

    with open(sp1_map_file, 'w') as f:
        for i in sp1_chr_map:
            f.write("%s\t%s\n" % (i, sp1_chr_map[i]))

    if species2_gff3:
        sp2_map_file = work_dir + "/" + species2_prefix + ".map"

        with open(sp2_map_file, 'w') as f:
            for i in sp2_chr_map:
                f.write("%s\t%s\n" % (i, sp2_chr_map[i]))


# results parser
class GeneLoci(object):
    def __init__(self, gene_id, chr_id, loci, species, gf):
        self.species = species
        self.id = gene_id
        self.loci = loci
        self.chr_id = chr_id
        self.gf = gf

    def __str__(self):
        return "%s: No. %d gene on %s from %s" % (self.id, self.loci, self.chr_id, self.species)


class Genome(object):
    def __init__(self, species_prefix, gff3_file=None, fasta_file=None):
        self.chr_dict = {}
        self.gene_dict = {}
        self.chr_length_dict = {}
        self.id = species_prefix

        if gff3_file:
            gff_dict = read_gff_file(gff3_file)

            chr_dict = {}
            gene_dict = {}
            for i in gff_dict['gene']:
                gf = gff_dict['gene'][i]
                if not gf.chr_id in chr_dict:
                    chr_dict[gf.chr_id] = []
                chr_dict[gf.chr_id].append(gf)
                gene_dict[gf.id] = gf

            for chr_id in chr_dict:
                chr_dict[chr_id] = sorted(
                    chr_dict[chr_id], key=lambda x: x.start)

            chr_gene_id_dict = {}
            for chr_id in chr_dict:
                chr_gene_id_dict[chr_id] = [i.id for i in chr_dict[chr_id]]

            self.chr_dict = {}
            self.gene_dict = {}
            for chr_id in chr_gene_id_dict:
                num = 0
                self.chr_dict[chr_id] = OrderedDict()
                for gene_id in chr_gene_id_dict[chr_id]:
                    gene = GeneLoci(gene_id, chr_id, num,
                                    species_prefix, gene_dict[gene_id])
                    self.chr_dict[chr_id][num] = gene
                    self.gene_dict[gene_id] = gene
                    num += 1

            self.chr_length_dict = {}
            if fasta_file:
                fa_dict = read_fasta_by_faidx(fasta_file)
                self.chr_length_dict = {i: fa_dict[i].len() for i in fa_dict}


class GenePair(object):
    def __init__(self, q_gene, s_gene, property_dict):
        self.q_gene = q_gene
        self.s_gene = s_gene
        self.property = property_dict

    def __str__(self):
        return "%s vs %s" % (self.q_gene, self.s_gene)

    def reverse_myself(self):
        new_GP = GenePair(self.s_gene, self.q_gene, self.property)
        return new_GP


class SyntenyBlock(object):
    def __init__(self, sb_id, q_sp, s_sp, strand, gene_pair_dict, property_dict, parameter_dict):
        self.id = sb_id
        self.property = property_dict
        self.parameter = parameter_dict
        self.strand = strand
        self.q_sp = q_sp
        self.s_sp = s_sp
        self.gene_pair_dict = gene_pair_dict

    def get_full_info(self, q_genome, s_genome):
        self.q_chr = self.gene_pair_dict[0].q_gene.chr_id
        self.s_chr = self.gene_pair_dict[0].s_gene.chr_id

        first_q_gene_loci = min(
            [self.gene_pair_dict[i].q_gene.loci for i in self.gene_pair_dict])
        last_q_gene_loci = max(
            [self.gene_pair_dict[i].q_gene.loci for i in self.gene_pair_dict])
        first_s_gene_loci = min(
            [self.gene_pair_dict[i].s_gene.loci for i in self.gene_pair_dict])
        last_s_gene_loci = max(
            [self.gene_pair_dict[i].s_gene.loci for i in self.gene_pair_dict])

        self.query_gene_list = []
        for i in range(first_q_gene_loci, last_q_gene_loci + 1):
            self.query_gene_list.append(q_genome.chr_dict[self.q_chr][i])

        self.subject_gene_list = []
        for i in range(first_s_gene_loci, last_s_gene_loci + 1):
            self.subject_gene_list.append(s_genome.chr_dict[self.s_chr][i])

        self.first_q_gene = q_genome.chr_dict[self.q_chr][min(
            [i.loci for i in self.query_gene_list])]
        self.last_q_gene = q_genome.chr_dict[self.q_chr][max(
            [i.loci for i in self.query_gene_list])]
        self.first_s_gene = s_genome.chr_dict[self.s_chr][min(
            [i.loci for i in self.subject_gene_list])]
        self.last_s_gene = s_genome.chr_dict[self.s_chr][max(
            [i.loci for i in self.subject_gene_list])]

        self.first_q_gene_loci = self.first_q_gene.loci
        self.last_q_gene_loci = self.last_q_gene.loci
        self.first_s_gene_loci = self.first_s_gene.loci
        self.last_s_gene_loci = self.last_s_gene.loci

        self.query_from = min([q_genome.gene_dict[self.first_q_gene.id].gf.start, q_genome.gene_dict[self.first_q_gene.id].gf.end,
                               q_genome.gene_dict[self.last_q_gene.id].gf.start, q_genome.gene_dict[self.last_q_gene.id].gf.end])
        self.query_to = max([q_genome.gene_dict[self.first_q_gene.id].gf.start, q_genome.gene_dict[self.first_q_gene.id].gf.end,
                             q_genome.gene_dict[self.last_q_gene.id].gf.start, q_genome.gene_dict[self.last_q_gene.id].gf.end])
        self.subject_from = min([s_genome.gene_dict[self.first_s_gene.id].gf.start, s_genome.gene_dict[self.first_s_gene.id].gf.end,
                                 s_genome.gene_dict[self.last_s_gene.id].gf.start, s_genome.gene_dict[self.last_s_gene.id].gf.end])
        self.subject_to = max([s_genome.gene_dict[self.first_s_gene.id].gf.start, s_genome.gene_dict[self.first_s_gene.id].gf.end,
                               s_genome.gene_dict[self.last_s_gene.id].gf.start, s_genome.gene_dict[self.last_s_gene.id].gf.end])

    def reverse_myself(self, new_sb_id=None):
        gene_pair_dict = {
            i: self.gene_pair_dict[i].reverse_myself() for i in self.gene_pair_dict}
        if new_sb_id is None:
            new_sb_id = self.id

        new_sb = SyntenyBlock(new_sb_id, self.s_sp, self.q_sp,
                              self.strand, gene_pair_dict, self.property, self.parameter)

        new_sb.q_chr = new_sb.gene_pair_dict[0].q_gene.chr_id
        new_sb.s_chr = new_sb.gene_pair_dict[0].s_gene.chr_id

        new_sb.query_gene_list = self.subject_gene_list
        new_sb.subject_gene_list = self.query_gene_list

        new_sb.first_q_gene = self.first_s_gene
        new_sb.last_q_gene = self.last_s_gene
        new_sb.first_s_gene = self.first_q_gene
        new_sb.last_s_gene = self.last_q_gene

        new_sb.first_q_gene_loci = new_sb.first_q_gene.loci
        new_sb.last_q_gene_loci = new_sb.last_q_gene.loci
        new_sb.first_s_gene_loci = new_sb.first_s_gene.loci
        new_sb.last_s_gene_loci = new_sb.last_s_gene.loci

        new_sb.query_from = self.subject_from
        new_sb.query_to = self.subject_to
        new_sb.subject_from = self.query_from
        new_sb.subject_to = self.query_to

        return new_sb

    def __str__(self):
        return "Q = %s:%s gene: %d-%d (%d) base: %d-%d (%d) vs S = %s:%s gene: %d-%d (%d) base: %d-%d (%d), %s, have %d gene pair" % (self.q_sp, self.q_chr, self.first_q_gene_loci, self.last_q_gene_loci, self.last_q_gene_loci - self.first_q_gene_loci + 1,  self.query_from, self.query_to, self.query_to - self.query_from + 1, self.s_sp, self.s_chr, self.first_s_gene_loci, self.last_s_gene_loci, self.last_s_gene_loci - self.first_s_gene_loci + 1,   self.subject_from, self.subject_to, self.subject_to - self.subject_from + 1, self.strand, len(self.gene_pair_dict))

    __repr__ = __str__


def sb_plot(sb):

    all_q_list = sb.query_gene_list
    all_s_list = sb.subject_gene_list

    fig, ax = plt.subplots(figsize=(10,20))

    vertical = -3
    start_site = 1
    ax.text(vertical, start_site-2, all_q_list[0].chr_id, ha='center', va='center')
    for q_gf in all_q_list:
        add_box(ax, start_site, start_site+1, vertical=vertical)
        q_gf.fig_site = (vertical, start_site)
        ax.text(vertical-1, start_site+1, q_gf.loci, ha='center', va='center')
        start_site += 3
    q_max_s = start_site

    vertical = 3
    start_site = 1
    ax.text(vertical, start_site-2, all_s_list[0].chr_id, ha='center', va='center')
    for s_gf in all_s_list:
        add_box(ax, start_site, start_site+1, vertical=vertical)
        s_gf.fig_site = (vertical, start_site)
        ax.text(vertical+1, start_site+1, s_gf.loci, ha='center', va='center')
        start_site += 3
    s_max_s = start_site    

    for gp_id in sb.gene_pair_dict:
        gp = sb.gene_pair_dict[gp_id]
        q_s = gp.q_gene.fig_site
        s_s = gp.s_gene.fig_site
        ax.plot([q_s[0], s_s[0]], [q_s[1], s_s[1]], color='k', alpha=0.8)

    ax.set_xlim(-10, 10)
    ax.set_ylim(-10, max(q_max_s, s_max_s)+2)    

    plt.gca().invert_yaxis()
    plt.axis('off')
    plt.show()    


def get_mcscan_parameter(mcscanx_collinearity_file):

    with open(mcscanx_collinearity_file, 'r') as f:
        for each_line in f:
            # statistics
            mobj = re.match(
                r"# Number of collinear genes: (\d+), Percentage: (\d+\.\d+)", each_line)
            if mobj:
                gene_in_coll, percentage = mobj.groups()
                gene_in_coll, percentage = int(gene_in_coll), float(percentage)

            mobj = re.match(r"# Number of all genes: (\d+)", each_line)
            if mobj:
                all_gene = mobj.groups()[0]
                all_gene = int(all_gene)

            # Parameters
            mobj = re.match(r"# MATCH_SCORE: (\S+)$", each_line)
            if mobj:
                MATCH_SCORE = mobj.groups()[0]
                MATCH_SCORE = int(MATCH_SCORE)

            mobj = re.match(r"# MATCH_SIZE: (\S+)$", each_line)
            if mobj:
                MATCH_SIZE = mobj.groups()[0]
                MATCH_SIZE = int(MATCH_SIZE)

            mobj = re.match(r"# GAP_PENALTY: (\S+)$", each_line)
            if mobj:
                GAP_PENALTY = mobj.groups()[0]
                GAP_PENALTY = int(GAP_PENALTY)

            mobj = re.match(r"# OVERLAP_WINDOW: (\S+)$", each_line)
            if mobj:
                OVERLAP_WINDOW = mobj.groups()[0]
                OVERLAP_WINDOW = int(OVERLAP_WINDOW)

            mobj = re.match(r"# E_VALUE: (\S+)$", each_line)
            if mobj:
                E_VALUE = mobj.groups()[0]
                E_VALUE = float(E_VALUE)

            mobj = re.match(r"# MAX GAPS: (\S+)$", each_line)
            if mobj:
                MAX_GAPS = mobj.groups()[0]
                MAX_GAPS = int(MAX_GAPS)

    parameter_dict = {
        "gene_in_coll": gene_in_coll,
        "percentage": percentage,
        "all_gene": all_gene,
        "match_score": MATCH_SCORE,
        "match_size": MATCH_SIZE,
        "gap_penalty": GAP_PENALTY,
        "overlap_window": OVERLAP_WINDOW,
        "e_value": E_VALUE,
        "max_gaps": MAX_GAPS
    }

    return parameter_dict


def collinearity_parser(mcscanx_collinearity_file, query_species, q_genome, subject_species, s_genome):
    mcscan_parameter = get_mcscan_parameter(mcscanx_collinearity_file)

    synteny_block_dict = {}

    with open(mcscanx_collinearity_file, 'r') as f:
        for each_line in f:
            # Block title
            mobj = re.match(
                r"## Alignment (\S+): score=(\S+) e_value=(\S+) N=(\S+) (\S+)&(\S+) (\S+)", each_line)
            if mobj:
                align_id, score, e_value, gene_pair_num, q_chr, s_chr, strand = mobj.groups()

                q_sp = re.sub(r'\d+', '', q_chr)
                s_sp = re.sub(r'\d+', '', s_chr)

                if q_sp != query_species or s_sp != subject_species:
                    continue

                align_id, score, e_value, gene_pair_num, q_chr, s_chr, strand = align_id, float(
                    score), float(e_value), int(gene_pair_num), q_chr, s_chr, strand
                if strand == 'plus':
                    strand = "+"
                elif strand == 'minus':
                    strand = "-"
                else:
                    raise

                property_dict = {
                    'score': score,
                    'e_value': e_value,
                    'gene_pair_num': gene_pair_num,
                }

                synteny_block_dict[align_id] = SyntenyBlock(
                    align_id, q_sp, s_sp, strand, {}, property_dict, mcscan_parameter)

            # block line
            if re.match("^#", each_line):
                continue
            else:
                if align_id not in synteny_block_dict:
                    continue

                align_id = each_line.split("-", 1)[0]
                pair_id = each_line.split("-", 1)[1].split(":", 1)[0]

                align_id = re.sub(r'\s+', '', align_id)
                pair_id = int(re.sub(r'\s+', '', pair_id))

                q_gene_id, s_gene_id, e_value = each_line.split(
                    "-", 1)[1].split(":", 1)[1].split()
                align_id, pair_id, q_gene_id, s_gene_id, e_value = align_id, pair_id, q_gene_id, s_gene_id, float(
                    e_value)

                q_gene = q_genome.gene_dict[q_gene_id]
                s_gene = s_genome.gene_dict[s_gene_id]

                property_dict = {'e_value': e_value}

                synteny_block_dict[align_id].gene_pair_dict[pair_id] = GenePair(
                    q_gene, s_gene, property_dict)

    for align_id in synteny_block_dict:
        synteny_block_dict[align_id].get_full_info(q_genome, s_genome)

    return synteny_block_dict


def pair_self_synteny_block_dict(synteny_block_dict, genome):
    full_synteny_block_dict = {
        i: synteny_block_dict[i] for i in synteny_block_dict}
    num = max(synteny_block_dict.keys())
    for i in synteny_block_dict:
        num += 1
        new_sb = synteny_block_dict[i].reverse_myself(num)
        new_sb.get_full_info(genome, genome)

        full_synteny_block_dict[num] = new_sb

    return full_synteny_block_dict


def get_OG_pair_set(OG_tsv_file, sp1, sp2=None):
    OG_dict = OG_tsv_file_parse(OG_tsv_file)

    OG_pair = []
    for OG_id in OG_dict:
        if sp2:
            sp1_list = OG_dict[OG_id][sp1]
            sp2_list = OG_dict[OG_id][sp2]

            if sp1_list == '' or sp2_list == '':
                continue

            for i in sp1_list:
                for j in sp2_list:
                    OG_pair.append((i, j))
        else:
            sp1_list = OG_dict[OG_id][sp1]

            if sp1_list == '':
                continue

            for i in sp1_list:
                for j in sp1_list:
                    if i == j:
                        continue
                    OG_pair.append((i, j))

    OG_pair = set(OG_pair)

    return OG_pair


def map_ortho_to_block(synteny_block, OG_pair_set):
    OG_pair_list = []

    for gp_id in synteny_block.gene_pair_dict:
        q_id = synteny_block.gene_pair_dict[gp_id].q_gene.id
        s_id = synteny_block.gene_pair_dict[gp_id].s_gene.id
        if (q_id, s_id) in OG_pair_set or (s_id, q_id) in OG_pair_set:
            OG_pair_list.append(gp_id)

    synteny_block.property['OG_pair_list'] = OG_pair_list

    return synteny_block


def get_synteny_block_interlap(synteny_block_dict):
    query_synteny_block_chr_interlap_dict = {}
    subject_synteny_block_chr_interlap_dict = {}
    for sb_id in synteny_block_dict:
        sb = synteny_block_dict[sb_id]
        q_chr = sb.q_chr
        s_chr = sb.s_chr
        query_synteny_block_chr_interlap_dict[q_chr] = InterLap()
        subject_synteny_block_chr_interlap_dict[s_chr] = InterLap()

    for sb_id in synteny_block_dict:
        sb = synteny_block_dict[sb_id]
        q_chr = sb.q_chr
        s_chr = sb.s_chr

        query_synteny_block_chr_interlap_dict[q_chr].add(
            (sb.first_q_gene_loci, sb.last_q_gene_loci, sb_id))
        subject_synteny_block_chr_interlap_dict[s_chr].add(
            (sb.first_s_gene_loci, sb.last_s_gene_loci, sb_id))

    return query_synteny_block_chr_interlap_dict, subject_synteny_block_chr_interlap_dict


def get_synteny_block_genome_range_interlap(synteny_block_dict):
    query_synteny_block_chr_interlap_dict = {}
    subject_synteny_block_chr_interlap_dict = {}
    for sb_id in synteny_block_dict:
        sb = synteny_block_dict[sb_id]
        q_chr = sb.q_chr
        s_chr = sb.s_chr
        query_synteny_block_chr_interlap_dict[q_chr] = InterLap()
        subject_synteny_block_chr_interlap_dict[s_chr] = InterLap()

    for sb_id in synteny_block_dict:
        sb = synteny_block_dict[sb_id]
        q_chr = sb.q_chr
        s_chr = sb.s_chr

        query_synteny_block_chr_interlap_dict[q_chr].add(
            (sb.first_q_gene.gf.start, sb.last_q_gene.gf.end, sb_id))
        subject_synteny_block_chr_interlap_dict[s_chr].add(
            (sb.first_s_gene.gf.start, sb.last_s_gene.gf.end, sb_id))

    return query_synteny_block_chr_interlap_dict, subject_synteny_block_chr_interlap_dict


def get_overlaped_block_group(synteny_block_dict):
    q_sb_interlap, s_sb_interlap = get_synteny_block_interlap(
        synteny_block_dict)

    group_list = []
    for sb_id in synteny_block_dict:
        group_list.append([sb_id])
        sb = synteny_block_dict[sb_id]

        q_over_list = [i[2] for i in q_sb_interlap[sb.q_chr].find(
            (sb.first_q_gene_loci, sb.last_q_gene_loci))]
        s_over_list = [i[2] for i in s_sb_interlap[sb.s_chr].find(
            (sb.first_s_gene_loci, sb.last_s_gene_loci))]

        overlap_list = list(set(q_over_list) & set(s_over_list))

        if len(overlap_list) > 0:
            # print(overlap_list)
            group_list.append(overlap_list)

    merged_group_list = merge_same_element_set(group_list)

    return merged_group_list


def get_merged_block(merge_id, group_list, synteny_block_dict, q_genome, s_genome):
    tmp_sb = synteny_block_dict[sorted(group_list, key=lambda x:(abs(
        synteny_block_dict[x].query_to - synteny_block_dict[x].query_from)), reverse=True)[0]]
    q_sp = tmp_sb.q_sp
    s_sp = tmp_sb.s_sp
    strand = tmp_sb.strand
    parameter_dict = tmp_sb.parameter

    gene_pair_dict = {}
    num = 0
    for i in group_list:
        for j in synteny_block_dict[i].gene_pair_dict:
            gene_pair_dict[num] = synteny_block_dict[i].gene_pair_dict[j]
            num += 1

    super_sb = SyntenyBlock(merge_id, q_sp, s_sp, strand,
                            gene_pair_dict, {}, parameter_dict)

    super_sb.get_full_info(q_genome, s_genome)

    return super_sb


def merge_blocks(synteny_block_dict, q_genome, s_genome):
    merged_group_list = get_overlaped_block_group(synteny_block_dict)
    num = 0
    merged_synteny_block_dict = {}
    for group_list in merged_group_list:
        merged_synteny_block_dict[num] = get_merged_block(
            num, group_list, synteny_block_dict, q_genome, s_genome)
        num += 1
    return merged_synteny_block_dict


def gene_cover_depth_stat(synteny_block_dict, query_or_subject, covered_genome):
    q_sb_interlap, s_sb_interlap = get_synteny_block_interlap(
        synteny_block_dict)

    if query_or_subject == 'query':
        sb_interlap = q_sb_interlap
    elif query_or_subject == 'subject':
        sb_interlap = s_sb_interlap

    # gene cover dict
    gene_cover_depth_dict = {}
    for g_id in covered_genome.gene_dict:
        gene = covered_genome.gene_dict[g_id]
        if gene.chr_id in sb_interlap:
            gene_cover_depth_dict[g_id] = len(
                list(sb_interlap[gene.chr_id].find((gene.loci, gene.loci))))
        else:
            gene_cover_depth_dict[g_id] = 0

    # range cover
    range_loci_cover_chr_dict = {}
    for chr_id in covered_genome.chr_dict:
        range_loci_cover_chr_dict[chr_id] = {}
        for gene_num in covered_genome.chr_dict[chr_id]:
            g = covered_genome.chr_dict[chr_id][gene_num]
            g_depth = gene_cover_depth_dict[g.id]
            if g_depth == 0:
                continue
            if g_depth not in range_loci_cover_chr_dict[chr_id]:
                range_loci_cover_chr_dict[chr_id][g_depth] = []
            range_loci_cover_chr_dict[chr_id][g_depth].append(
                (gene_num, gene_num))
            range_loci_cover_chr_dict[chr_id][g_depth] = merge_intervals(
                range_loci_cover_chr_dict[chr_id][g_depth], True)

    range_base_cover_chr_dict = {}
    for chr_id in range_loci_cover_chr_dict:
        range_base_cover_chr_dict[chr_id] = {}
        for depth in range_loci_cover_chr_dict[chr_id]:
            if depth == 0:
                continue
            range_base_cover_chr_dict[chr_id][depth] = []

            for s, e in range_loci_cover_chr_dict[chr_id][depth]:
                start = covered_genome.chr_dict[chr_id][s].gf.start
                end = covered_genome.chr_dict[chr_id][e].gf.end
                range_base_cover_chr_dict[chr_id][depth].append((start, end))

    return gene_cover_depth_dict, range_loci_cover_chr_dict, range_base_cover_chr_dict


def WGD_check_pipeline(mcscanx_collinearity_file, query_species, query_gff3, query_genome_fasta, subject_species, subject_gff3, subject_genome_fasta, OG_tsv_file=None, OG_filter=0.3):
    q_genome = Genome(query_species, query_gff3, query_genome_fasta)
    s_genome = Genome(subject_species, subject_gff3, subject_genome_fasta)

    # load block
    synteny_block_dict = collinearity_parser(
        mcscanx_collinearity_file, query_species, q_genome, subject_species, s_genome)
    # print(len(synteny_block_dict))

    # filter by OG
    if OG_tsv_file:
        OG_pair_set = get_OG_pair_set(
            OG_tsv_file, query_species, subject_species)

        for i in synteny_block_dict:
            synteny_block_dict[i] = map_ortho_to_block(
                synteny_block_dict[i], OG_pair_set)

        synteny_block_dict = {i: synteny_block_dict[i] for i in synteny_block_dict if len(
            synteny_block_dict[i].property['OG_pair_list']) / len(synteny_block_dict[i].gene_pair_dict) > OG_filter}

    # merge block
    synteny_block_dict = merge_blocks(synteny_block_dict, q_genome, s_genome)

    # get gene cover dict
    q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict = gene_cover_depth_stat(
        synteny_block_dict, 'query', q_genome)
    s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict = gene_cover_depth_stat(
        synteny_block_dict, 'subject', s_genome)

    return q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict, s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict, synteny_block_dict


def WGD_check_pipeline_for_self_comparsion(mcscanx_collinearity_file, species_id, gff3_file, genome_fasta, OG_tsv_file=None, OG_filter=0.3):
    genome = Genome(species_id, gff3_file, genome_fasta)

    # load block
    synteny_block_dict = collinearity_parser(
        mcscanx_collinearity_file, species_id, genome, species_id, genome)
    # print(len(synteny_block_dict))

    # filter by OG
    if OG_tsv_file:
        OG_pair_set = get_OG_pair_set(OG_tsv_file, species_id)

        for i in synteny_block_dict:
            synteny_block_dict[i] = map_ortho_to_block(
                synteny_block_dict[i], OG_pair_set)

        synteny_block_dict = {i: synteny_block_dict[i] for i in synteny_block_dict if len(
            synteny_block_dict[i].property['OG_pair_list']) / len(synteny_block_dict[i].gene_pair_dict) > OG_filter}

    # merge block
    synteny_block_dict = merge_blocks(synteny_block_dict, genome, genome)
    synteny_block_dict = pair_self_synteny_block_dict(
        synteny_block_dict, genome)

    # get gene cover dict
    gene_cover_depth_dict, range_loci_cover_chr_dict, range_base_cover_chr_dict = gene_cover_depth_stat(
        synteny_block_dict, 'query', genome)

    return gene_cover_depth_dict, range_loci_cover_chr_dict, range_base_cover_chr_dict, synteny_block_dict


def collinearity_file_to_synteny_block_range_csv(query_id, query_gff, subject_id, subject_gff, mcscanx_collinearity_file, csv_file):

    q_genome = Genome(query_id, query_gff)
    s_genome = Genome(subject_id, subject_gff)

    # load block
    synteny_block_dict = collinearity_parser(
        mcscanx_collinearity_file, query_id, q_genome, subject_id, s_genome)

    with open(csv_file, 'w') as f:

        for sb_id in synteny_block_dict:
            sb = synteny_block_dict[sb_id]

            sb.query_from
            sb.query_to
            sb.subject_from
            sb.subject_to
            sb.strand

            f.write(printer_list([query_id, sb.q_chr, sb.query_from, sb.query_to,
                                  subject_id, sb.s_chr, sb.subject_from, sb.subject_to, sb.strand]) + "\n")


if __name__ == "__main__":

    species1_prefix = 'Aof'
    species1_gff3 = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Other/Aof.merge.gff3'
    species1_aa_fasta = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Other/Aof.merge.fasta'

    species2_prefix = 'Gel'
    species2_gff3 = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Other/Gel.merge.gff3'
    species2_aa_fasta = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Other/Gel.merge.fasta'

    work_dir = '/lustre/home/xuyuxing/Work/Gel/mcscanx/Gel_vs_Other/python'
    give_bls_file = None
    skip_gene_list = []

    running_mcscanx(species1_gff3, species2_gff3, work_dir, species1_prefix='aa', species1_aa_fasta=None,
                    species2_prefix='bb', species2_aa_fasta=None, skip_gene_list=[], give_bls_file=None)

    # parse
    # from lib.evolution.mcscan import WGD_check_pipeline
    from collections import Counter, OrderedDict

    query_species = 'Ash'
    query_gff3 = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Ash.gff3'
    query_genome_fasta = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Apostasia_shenzhenica/T1088818N0.genome.fasta'
    subject_species = 'Gel'
    subject_gff3 = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Gel.gff3'
    subject_genome_fasta = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/Gel.genome.v2.0.fasta'
    mcscanx_collinearity_file = '/lustre/home/xuyuxing/Work/Gel/synteny/redo_big/Gel_vs_Ash/mcscanx.collinearity'

    q_gene_covered_dict, q_range_loci_cover_chr_dict, q_range_base_cover_chr_dict, s_gene_covered_dict, s_range_loci_cover_chr_dict, s_range_base_cover_chr_dict, synteny_block_dict = WGD_check_pipeline(
        mcscanx_collinearity_file, query_species, query_gff3, query_genome_fasta, subject_species, subject_gff3, subject_genome_fasta)

    Counter([q_gene_covered_dict[i] for i in q_gene_covered_dict])
    Counter([s_gene_covered_dict[i] for i in s_gene_covered_dict])



from lib.common.fileIO import tsv_file_dict_parse
import re
from api.xuyuxing.comp_genome.orthofinder import OG_tsv_file_parse

class InterproRecord(object):
    def __init__(self, seqID, domain_list):
        self.seqID = seqID
        self.domain_list = domain_list

    def all_analysis(self):
        all_analysis_list = []
        for i in self.domain_list:
            all_analysis_list.append(i["Analysis"])
        all_analysis_list = list(set(all_analysis_list))
        return all_analysis_list

    def domain_from_analysis(self, analysis_name):
        select_domain_list = []
        for i in self.domain_list:
            if i["Analysis"] == analysis_name:
                select_domain_list.append(i)
        return select_domain_list

    def get_go(self):
        GO_list = []
        for domain in self.domain_list:
            if domain['GO'] is None or domain['GO'] == '':
                continue
            GO_list.extend(domain['GO'].split("|"))

        GO_list = list(set(GO_list))

        self.go = GO_list

        return GO_list




column_list = ["query", "MD5", "Length", "Analysis", "Accession", "Description", "Start", "Stop", "Score", "Status",
               "Date", "InterPro_accession", "InterPro_annotations", "GO", "Pathways"]


def interproscan_results_parser(tsv_file):
    output = tsv_file_dict_parse(tsv_file, fieldnames=column_list)
    query_dict = {}
    for id_tmp in output:
        info = output[id_tmp]
        if info["query"] not in query_dict:
            query_dict[info["query"]] = InterproRecord(info["query"], [])
        query_dict[info["query"]].domain_list.append(info)
    return query_dict


def parse_entry_file(entry_file):

    tmp_dict = tsv_file_dict_parse(entry_file, key_col='ENTRY_AC')
    output_dict = {}
    for i in tmp_dict:
        if tmp_dict[i]['ENTRY_TYPE'] not in output_dict:
            output_dict[tmp_dict[i]['ENTRY_TYPE']] = {}
        output_dict[tmp_dict[i]['ENTRY_TYPE']][tmp_dict[i]
                                               ['ENTRY_AC']] = tmp_dict[i]['ENTRY_NAME']

    return output_dict


def parse_interpro2go(interpro2go_file):
    ip2go_dict = []
    with open(interpro2go_file, 'r') as f:
        for each_line in f:
            each_line = each_line.strip()
            ip = re.findall('IPR\d\d\d\d\d\d', each_line)[0]
            go = re.findall('GO:\d\d\d\d\d\d\d', each_line)[0]

            if ip not in ip2go_dict:
                ip2go_dict[ip] = []

            ip2go_dict[ip].append(go)

            ip2go_dict[ip] = list(set(ip2go_dict[ip]))

    return ip2go_dict


def get_orthogroup_GO(Orthogroups_tsv, gene_interpro_tsv):
    OG_dict = OG_tsv_file_parse(Orthogroups_tsv)
    gene_interpro_dict = interproscan_results_parser(gene_interpro_tsv)
    
    GO_dict = {}
    for OG_id in OG_dict:
        GO_list = []
        for speci in OG_dict[OG_id]:
            for gene in OG_dict[OG_id][speci]:
                if gene == '':
                    continue
                if gene in gene_interpro_dict:
                    GO_list.extend(gene_interpro_dict[gene].get_go())
        GO_list = list(set(GO_list))
        GO_dict[OG_id] = GO_list
        
    return GO_dict    



if __name__ == "__main__":

    # %%
    # make interpro family report


    # wget ftp://ftp.ebi.ac.uk/pub/databases/interpro/entry.list
    interpro_entry_file = '/lustre/home/xuyuxing/Database/Interpro/entry.list'
    entry_dict = parse_entry_file(interpro_entry_file)

    # wget ftp://ftp.ebi.ac.uk/pub/databases/interpro/interpro2go
    interpro2go_file = '/lustre/home/xuyuxing/Database/Interpro/interpro2go'

    reference_genome_table = '/lustre/home/xuyuxing/Work/Gel/WPGmapper/Gel_ref.xlsx'
    specie_list = ["T55571N0", "T90708N0", "T1088818N0", "T4686N0", "T906689N0", "T91201N0", "T78828N0", "T13894N0", "T51953N0", "T4530N0", "T4577N0", "T4641N0"]

    import pandas as pd

    query_db = pd.read_excel(reference_genome_table)
    interpro_file_dict = {}
    for index in query_db.index:
        query_id = str(query_db.loc[index]['id'])
        interpro_file = str(query_db.loc[index]['function_anno'])

        if not pd.isna(query_id) and not pd.isna(interpro_file) and query_id in specie_list:
            interpro_file_dict[query_id] = interpro_file

    ip_gene_dict = {}

    for query_id in interpro_file_dict:
        print(query_id)

        interpro_file = interpro_file_dict[query_id]
        interpro_dict = interproscan_results_parser(interpro_file)

        ip_gene = {}

        for g_id in interpro_dict:
            for domain_dict in interpro_dict[g_id].domain_list:
                ip = domain_dict['InterPro_accession']
                if ip not in ip_gene:
                    ip_gene[ip] = []
                ip_gene[ip].append(g_id)
                ip_gene[ip] = list(set(ip_gene[ip]))
        
        ip_gene_dict[query_id] = ip_gene

    # ip_gene_count

    ip_list = []

    for query_id in ip_gene_dict:
        ip_list.extend(ip_gene_dict[query_id])

    ip_list = list(set(ip_list))

    ip_count_dict = {}

    for ip in ip_list:
        count_list = []
        for sp in specie_list:
            if ip in ip_gene_dict[sp]:
                count_list.append(len(ip_gene_dict[sp][ip]))
            else:
                count_list.append(0)
            ip_count_dict[ip] = count_list

    # output
    from lib.common.util import printer_list
    from api.xuyuxing.evolution.badirate import F_index


    entry_info_dict = {}
    for ip_type in entry_dict:
        for ip in entry_dict[ip_type]:
            entry_info_dict[ip] = (ip_type, entry_dict[ip_type][ip])

    with open("/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S12/Interpro.family.tsv", 'w') as f:

        header_list = ["Interpro AC", "Interpro Type", "Description"]
        header_list.extend(specie_list+specie_list)
        f.write(printer_list(header_list)+"\n")

        for ip in ip_count_dict:
            if ip not in entry_info_dict:
                continue

            f_index_tuple = F_index(ip_count_dict[ip])
            output_list = [ip, entry_info_dict[ip][0], entry_info_dict[ip][1]]
            output_list.extend(ip_count_dict[ip])
            output_list.extend(f_index_tuple)
            f.write(printer_list(output_list)+"\n")
    
    # %%
    # make orthogroup GO annotation

    # 
    Orthogroups_tsv = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S12/Orthogroups.tsv'
    gene_interpro_tsv = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S12/gene.interpro.tsv'
    output_file = '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S12/Orthogroups.interpro.tsv'

    GO_dict = get_orthogroup_GO(Orthogroups_tsv, gene_interpro_tsv)

    from lib.common.util import printer_list
    with open(output_file,'w') as f:
        for i in GO_dict:
            GO_list = GO_dict[i]
            if len(GO_list) == 0:
                continue
            GO_string = printer_list(GO_list,', ')
            f.write("%s\t%s\n" % (i,GO_string))




import re
from collections import OrderedDict
from lib.common.genome.seq_base import read_fasta_by_faidx, FastaRecord, write_fasta


def read_assembly(assembly_file):
    """
    assembly_file = '/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/Gel.genome.v1.0.final.assembly'
    """

    contig_dict = OrderedDict()
    scaffold_dict = OrderedDict()
    scaf_id = 0
    with open(assembly_file, 'r') as f:
        for each_line in f:
            each_line = each_line.strip()
            matchObj = re.match('^>(\S+) (\d+) (\d+)$', each_line)
            if matchObj:
                contig_name, ID_num, contig_length = matchObj.groups()

                contig_name = re.sub(r':::debris', '', contig_name)
                contig_name = re.sub(r':::', '_', contig_name)

                contig_dict[ID_num] = (contig_name, ID_num, contig_length)
            else:
                scaf_info = each_line.split()

                scaf_contig_id_list = []
                scaf_contig_length_list = []
                scaf_contig_strand_list = []
                for contig_id in scaf_info:
                    if '-' in contig_id:
                        scaf_contig_strand_list.append('-')
                    else:
                        scaf_contig_strand_list.append('+')
                    contig_id = re.sub(r'-', '', contig_id)
                    scaf_contig_id_list.append(contig_dict[contig_id][0])
                    scaf_contig_length_list.append(
                        int(contig_dict[contig_id][2]))

                scaffold_dict['scaf_%d' % scaf_id] = {
                    "contig_list": scaf_contig_id_list,
                    "contig_length": scaf_contig_length_list,
                    "contig_strand": scaf_contig_strand_list
                }
                scaf_id += 1

    return scaffold_dict


def write_assembly(scaffold_dict, assembly_file):
    """
    scaffold_dict['scaf_%d' % scaf_id] = {
        "contig_list": scaf_contig_id_list,
        "contig_length": scaf_contig_length_list,
        "contig_strand": scaf_contig_strand_list
    }  
    """

    contig_id_dict = {}
    contig_len_dict = {}
    contig_strand_dict = {}
    num = 1
    for i in scaffold_dict:
        for c_id, length, strand in zip(scaffold_dict[i]['contig_list'], scaffold_dict[i]['contig_length'], scaffold_dict[i]['contig_strand']):
            contig_id_dict[c_id] = num
            contig_len_dict[c_id] = length
            contig_strand_dict[c_id] = strand
            num += 1

    with open(assembly_file, 'w') as f:
        for i in contig_id_dict:
            f.write(">%s %d %d\n" % (i, contig_id_dict[i], contig_len_dict[i]))

        for i in scaffold_dict:
            print_list = []
            for c_id in scaffold_dict[i]['contig_list']:
                if contig_strand_dict[c_id] == '-':
                    print_list.append('-'+str(contig_id_dict[c_id]))
                else:
                    print_list.append(str(contig_id_dict[c_id]))

            f.write(" ".join(print_list)+"\n")


def assembly_to_fasta(contig_fasta, assembly_file, output_fasta):
    scaffold_dict = read_assembly(assembly_file)
    contig_fasta_dict = read_fasta_by_faidx(contig_fasta)

    num = 0
    fasta_record_list = []
    for i in scaffold_dict:

        scaffold_name = "HiC_scaffold_%d" % num
        scaffold_seq = ""

        for c_id, length, strand in zip(scaffold_dict[i]['contig_list'], scaffold_dict[i]['contig_length'], scaffold_dict[i]['contig_strand']):
            c_rec = contig_fasta_dict[c_id]
            if strand == '-':
                c_seq = c_rec.reverse_complement()
            else:
                c_seq = c_rec.seq

            if not scaffold_seq == "":
                scaffold_seq = scaffold_seq + "N" * 500

            scaffold_seq = scaffold_seq + c_seq

        fasta_record_list.append(FastaRecord(
            seqname=scaffold_name, seq=scaffold_seq))

        num += 1

    write_fasta(fasta_record_list, output_fasta, wrap_length=75, upper=True)



from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.xuyuxing.evolution.taxonomy import build_taxon_database
from collections import OrderedDict
from lib.common.util import printer_list
import re

def ContaminationFilter_main(args):
    split_genome = read_fasta_by_faidx(args.split_genome_fasta)
    split_contig_list = list(split_genome.keys())
    tax_record_dict = build_taxon_database(args.tax_dir)

    # read contig split
    contig_split_dict = {}
    for split_contig_id in split_contig_list:
        matchOBJ = re.match(r'^(\S+)_(\d+)-(\d+)$', split_contig_id)
        if matchOBJ:
            contig_id, start, end = matchOBJ.groups()
            if not contig_id in contig_split_dict:
                contig_split_dict[contig_id] = []
            contig_split_dict[contig_id].append(split_contig_id)

    # read taxon_assign_output
    from lib.common.fileIO import tsv_file_dict_parse

    taxon_assign_dict = tsv_file_dict_parse(args.taxon_assign_output,
                                            fieldnames=["split_id", "mrca_taxon_id", "mrca_taxon",
                                                        "recommand_taxon_id",
                                                        "recommand_taxon", "q_value"], key_col="split_id")

    classification_type = OrderedDict(
        {"Viruses": "10239", "Archaea": "2157", "Bacteria": "2", "Viridiplantae": "33090",
            "Fungi": "4751", "Metazoa": "33208", "Other": None, "No_hit": None})

    contig_taxon_dict = {}
    for contig_id in contig_split_dict:
        contig_taxon_dict[contig_id] = {i: 0 for i in classification_type}
        for split_id in contig_split_dict[contig_id]:
            if split_id in taxon_assign_dict:
                recommand_taxon_id = taxon_assign_dict[split_id]["recommand_taxon_id"]
                recommand_taxon = tax_record_dict[recommand_taxon_id]
                recommand_taxon_lineage = [i[0] for i in recommand_taxon.get_lineage(tax_record_dict)]
                recommand_classtype_list = [i for i in classification_type if
                                            classification_type[i] in recommand_taxon_lineage]
                if len(recommand_classtype_list) == 0:
                    contig_taxon_dict[contig_id]["Other"] += 1
                else:
                    recommand_classtype = recommand_classtype_list[0]
                    contig_taxon_dict[contig_id][recommand_classtype] += 1
            else:
                contig_taxon_dict[contig_id]["No_hit"] += 1

    with open(args.output_file, 'w') as f:
        header_string = printer_list([type for type in classification_type], sep="\t", head="contig_id\t")
        f.write(header_string + "\n")

        for contig_id in contig_taxon_dict:
            record_string = printer_list([contig_taxon_dict[contig_id][type] for type in classification_type],
                                            sep="\t", head=contig_id + "\t")
            f.write(record_string + "\n")

"""
genome_fasta=
split_genome_fasta=$genome_fasta.split
diamond_output=$split_genome_fasta.bls
taxon_assign_output=$split_genome_fasta.taxon
contamination_report=contamination_report.txt

taxon_nr_db=/lustre/home/xuyuxing/Database/NCBI/nr/2020/nr.taxon.dmnd
taxon_dump=/lustre/home/xuyuxing/Database/NCBI/nr/2020/taxdmp
threads=56
max_target_seqs=50

python ~/python_project/Genome_work_tools/GenomeTools.py FragmentGenome $genome_fasta $split_genome_fasta 10000 10000

diamond blastx --query $split_genome_fasta --max-target-seqs $max_target_seqs --db $taxon_nr_db --evalue 1e-5 --out $diamond_output --outfmt 6 qseqid sseqid staxids pident length mismatch gapopen qstart qend sstart send evalue bitscore --threads $threads

python ~/python_project/Genome_work_tools/TaxonTools.py DiamondTaxonAssign -n $threads -o $taxon_assign_output $diamond_output $taxon_dump

python ~/python_project/Genome_work_tools/GenomeTools.py ContaminationFilter $split_genome_fasta $taxon_assign_output $contamination_report
"""

"""
class abc():
    pass

args=abc()

args.split_genome_fasta = "/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/diamond/Gel.genome.v1.0.final.rename.split.fasta"
args.taxon_assign_output = "/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/diamond/Gel.genome.v1.0.final.rename.split.taxon"
args.tax_dir = '/lustre/home/xuyuxing/Database/NCBI/nr/2020/taxdmp'
args.output_file = "/lustre/home/xuyuxing/Database/Gel/genome/assembly/contig_filter/diamond/Gel.genome.v1.0.final.rename.contamination_report.txt"
"""


import sys
import re
import string
import commands
import pickle
import copy
import itertools
import subprocess


def samtools_to_fasta(range_string,genome_file_name,output_file):
	cmd_string = "samtools faidx "+genome_file_name+" \""+range_string+"\" >> \""+output_file+"\""
	print cmd_string
	p = subprocess.Popen(cmd_string, shell=True, stderr=subprocess.PIPE)
	p.wait()
	returncode = p.poll()
	if returncode == 1:
		subprocess.call("rm "+output_file, shell=True)
		print "error02"
		return "error02"
	else:
		return output_file
def phylogene_tree(seq_file,tag):
	cmd_string = "clustalw -INFILE=\""+seq_file+"\" -ALIGN -OUTPUT=FASTA -OUTFILE=\""+tag+".aln\" -type=protein"
	p = subprocess.Popen(cmd_string, shell=True, stderr=subprocess.PIPE)
	p.wait()
	returncode = p.poll()
	aln_file = tag+".aln"
	if returncode == 1:
		subprocess.call("rm "+aln_file, shell=True)
		print "error03"
		return "error03"
	cmd_string = "trimal -fasta -gappyout -in \""+aln_file+"\" -out \""+aln_file+".trimal\""
	p = subprocess.Popen(cmd_string, shell=True, stderr=subprocess.PIPE)
	p.wait()
	returncode = p.poll()
	trimal_file = aln_file+".trimal"
	if returncode == 1:
		subprocess.call("rm "+trimal_file, shell=True)
		print "error04"
		return "error04"
	cmd_string = "FastTree \""+trimal_file+"\" > \""+trimal_file+".phb\""
	p = subprocess.Popen(cmd_string, shell=True, stderr=subprocess.PIPE)
	p.wait()
	returncode = p.poll()
	phb_file = trimal_file+".phb"
	if returncode == 1:
		subprocess.call("rm "+trimal_file, shell=True)
		print "error05"
		return "error05"
	else:
		print "ML successful"
		return phb_file


def section(list1,list2):
	all = sorted(list(list1+list2))
	deta = all[2]-all[1]+1
	if max(list1)>=min(list2) and max(list2)>=min(list1):
		If_inter = 1	#Yes
	else:
		If_inter = 0	#No
	return int(If_inter),int(deta)


def hash_map(dict_list):
	dict_hash={}
	for i in dict_list:
		for j in dict_list[i]:
			if not dict_hash.has_key(j):
				dict_hash[j]=[]
			dict_hash[j].append(i)
	return dict_hash
def integrate_element(dict_list,num):
	dict_hash = hash_map(dict_list)
	flag = 0
	deleted=[]
	for i in dict_hash:
		if not len(dict_hash[i]) > 1:
			continue
		dict_list[num]=[]
		for j in dict_hash[i]:
			if j in deleted:
				continue
			dict_list[num]=list(set(dict_list[num]+dict_list[j]))
			del dict_list[j]
			deleted.append(j)
			flag=1
		num=num+1
		break
	return flag,num
def NoRed_element(list_all):
	dict_list={}
	num=0
	for i in list_all:
		dict_list[num]=i
		num=num+1
	#print dict_list
	flag = 1
	while flag == 1:
		flag,num = integrate_element(dict_list,num)
		print len(dict_list)
	output=[]
	for i in dict_list:
		output.append(dict_list[i])
	return output


def Parse_collinearity(mcscanx_collinearity_file):
	F2 = open(mcscanx_collinearity_file)
	colline_block={}
	block_id_dict={}
	for each_line in F2:
		if re.match(r'^#',each_line):
			match = re.match(r'## Alignment .* .* .* .* (\S\S)(.*)\&(\S\S)(.*) .*',each_line)
			if match:
				speci1=match.group(1)
				speci1_contig = match.group(2)
				speci2=match.group(3)
				speci2_contig = match.group(4)
				continue
			else:
				continue
		each_line = re.sub('\n','',each_line)
		info = string.split(each_line,"\t")
		info[0]=re.sub(' ','',info[0])
		block_id,pair_id=string.split(info[0],"-")
		pair_id = re.sub(':','',pair_id)
		
		speci_tag=tuple(sorted([speci1,speci2]))
		if not colline_block.has_key(speci_tag):
			colline_block[speci_tag]={}
		
		if not colline_block[speci_tag].has_key(block_id):
			colline_block[speci_tag][block_id]=[[],[]]
		
		if speci1 == speci2:
			colline_block[speci_tag][block_id][0].append(info[1])
			colline_block[speci_tag][block_id][1].append(info[2])
		elif speci_tag[0]==speci1:					  
			colline_block[speci_tag][block_id][0].append(info[1])
			colline_block[speci_tag][block_id][1].append(info[2])
		elif speci_tag[0]==speci2:					  
			colline_block[speci_tag][block_id][0].append(info[2])
			colline_block[speci_tag][block_id][1].append(info[1])
		
		block_id_dict[block_id]={}
		block_id_dict[block_id]['speci_tag']=speci_tag
		block_id_dict[block_id]['speci1_contig']=speci1_contig
		block_id_dict[block_id]['speci2_contig']=speci2_contig
	F2.close()
	return colline_block,block_id_dict
	
def Find_self_overlap_block(colline_block):
	colline_block_range={}
	colline_block_range_map={}
	for speci_tag in colline_block:
		colline_block_range[speci_tag]={}
		colline_block_range_map[speci_tag]={}
		colline_block_range_map[speci_tag][0]={}
		colline_block_range_map[speci_tag][1]={}
		for block_id in colline_block[speci_tag]:
			list1=[]
			list2=[]
			for i in range(0,len(colline_block[speci_tag][block_id][0])):
				gene1=colline_block[speci_tag][block_id][0][i]
				gene2=colline_block[speci_tag][block_id][1][i]
				list1=list1+[gene_vs_verything[gene1]['start'],gene_vs_verything[gene1]['end']]
				list2=list2+[gene_vs_verything[gene2]['start'],gene_vs_verything[gene2]['end']]
				min_range1=min(list1)
				max_range1=max(list1)
				min_range2=min(list2)
				max_range2=max(list2)
				speci1_contig=gene_vs_verything[gene1]['contig']
				speci2_contig=gene_vs_verything[gene2]['contig']
				speci1=gene_vs_verything[gene1]['speci']
				speci2=gene_vs_verything[gene2]['speci']
			speci1_contig_range = [min_range1,max_range1]
			speci2_contig_range = [min_range2,max_range2]
			colline_block_range[speci_tag][block_id]=[[speci1,speci1_contig,speci1_contig_range],[speci2,speci2_contig,speci2_contig_range]]
			if not colline_block_range_map[speci_tag][0].has_key(speci1_contig):
				colline_block_range_map[speci_tag][0][speci1_contig]=[]
			if not colline_block_range_map[speci_tag][1].has_key(speci2_contig):
				colline_block_range_map[speci_tag][1][speci2_contig]=[]
			colline_block_range_map[speci_tag][0][speci1_contig].append(block_id)
			colline_block_range_map[speci_tag][1][speci2_contig].append(block_id)
	
	intersection_dict={}
	intersection_dict['same_contig']=[]
	intersection_dict['overlap']=[]
	for speci_tag in colline_block_range_map:
		for contig1 in colline_block_range_map[speci_tag][0]:
			for contig2 in colline_block_range_map[speci_tag][1]:
				list1 = colline_block_range_map[speci_tag][0][contig1]
				list2 = colline_block_range_map[speci_tag][1][contig2]
				intersection = list(set(list1)&set(list2))
				if len(intersection) == 0:
					continue
				intersection_dict['same_contig'].append(intersection)
				for block_id1 in intersection:
					for block_id2 in intersection:
						if block_id1 == block_id2:
							continue
						block_id1_info=colline_block_range[speci_tag][block_id1]
						block_id2_info=colline_block_range[speci_tag][block_id2]
						speci1_flag,useless = section(block_id1_info[0][2],block_id2_info[0][2])
						speci2_flag,useless = section(block_id1_info[1][2],block_id2_info[1][2])
						if speci1_flag and speci2_flag:
							intersection_dict['overlap'].append([block_id1,block_id2])
	intersection_dict['overlap'] = NoRed_element(intersection_dict['overlap'])
	return colline_block_range,colline_block_range_map,intersection_dict


def Find_intra_overlap_block(colline_block_all,colline_block_standard):
	colline_block_all_range,colline_block_all_range_map,intersection_all_dict=Find_self_overlap_block(colline_block_all)
	colline_block_standard_range,colline_block_standard_range_map,intersection_standard_dict=Find_self_overlap_block(colline_block_standard)
	intra_overlap={}
	for speci_tag in colline_block_standard_range_map:
		speci1_intersection=[]
		for speci1_contig in colline_block_standard_range_map[speci_tag][0]:
			for standard_block_id in colline_block_standard_range_map[speci_tag][0][speci1_contig]:
				if not colline_block_all_range_map[speci_tag][0].has_key(speci1_contig):
					#print speci1_contig
					continue
				for all_block_id in colline_block_all_range_map[speci_tag][0][speci1_contig]:
					standard_block_info=colline_block_standard_range[speci_tag][standard_block_id]
					all_block_info=colline_block_all_range[speci_tag][all_block_id]
					speci1_flag,useless = section(standard_block_info[0][2],all_block_info[0][2])
					if speci1_flag:
						speci1_intersection.append((standard_block_id,all_block_id))
		speci2_intersection=[]
		for speci2_contig in colline_block_standard_range_map[speci_tag][1]:
			for standard_block_id in colline_block_standard_range_map[speci_tag][1][speci2_contig]:
				if not colline_block_all_range_map[speci_tag][1].has_key(speci2_contig):
					#print speci2_contig
					continue
				for all_block_id in colline_block_all_range_map[speci_tag][1][speci2_contig]:
					standard_block_info=colline_block_standard_range[speci_tag][standard_block_id]
					all_block_info=colline_block_all_range[speci_tag][all_block_id]
					speci2_flag,useless = section(standard_block_info[1][2],all_block_info[1][2])
					if speci2_flag:
						speci2_intersection.append((standard_block_id,all_block_id))
		intra_overlap[speci_tag]=list(set(speci1_intersection) & set(speci2_intersection))
	return intra_overlap
		
def block_search_range(string_search,colline_block_range,colline_block_range_map):
	#string_search="Cc:chr1:1000-2000"
	matchObj=re.match(r'(.*):(.*):(\d+)-(\d+)',string_search)
	output=[]
	if matchObj:
		speci = matchObj.group(1)
		contig = matchObj.group(2)
		start = int(matchObj.group(3))
		end = int(matchObj.group(4))
		#print speci,contig,start,end
		for speci_tag in colline_block_range_map:
			if not speci in speci_tag:
				continue
			speci_index = speci_tag.index(speci)
			if not colline_block_range_map[speci_tag][speci_index].has_key(contig):
				continue
			for block_id in colline_block_range_map[speci_tag][speci_index][contig]:
				info = colline_block_range[speci_tag][block_id]
				speci_info,speci_contig_info,speci_contig_range = info[speci_index]
		#		print speci,speci_info
		#		print contig,speci_contig_info
				speci_flag,useless = section(speci_contig_range,[start,end])
				if speci_flag:
					output.append(block_id)
		return output
	else:
		print "Bad string_search"
		return
	
def block_search_gene(gene1,gene2,gene_vs_verything,colline_block_range,colline_block_range_map):
	gene1_speci = gene_vs_verything[gene1]['speci']
	gene1_contig = gene_vs_verything[gene1]['contig']
	gene1_start = gene_vs_verything[gene1]['start']
	gene1_end = gene_vs_verything[gene1]['end']
	gene2_speci = gene_vs_verything[gene2]['speci']
	gene2_contig = gene_vs_verything[gene2]['contig']
	gene2_start = gene_vs_verything[gene2]['start']
	gene2_end = gene_vs_verything[gene2]['end']
	
	if not gene1_speci == gene2_speci:
		print "Bad querys"
		return
		
	if not gene1_contig == gene2_contig:
		print "Bad querys"
		return
	
	range_list = [gene1_start,gene1_end,gene2_start,gene2_end]


	string_search = gene1_speci+":"+gene1_contig+":"+str(min(range_list))+"-"+str(max(range_list))
	#print string_search
	output = block_search_range(string_search,colline_block_range,colline_block_range_map)
	return output
	
F2 = open("mcscanx.gff")
gff_speci_vs_contig_vs_gene={}
gene_vs_verything={}
for each_line in F2:
	each_line = re.sub('\n','',each_line)
	info = string.split(each_line,"\t")
	match = re.match(r'(\S\S)(.*)',info[0])
	speci=match.group(1)
	contig=match.group(2)
	gene = info[1]
	start = int(info[2])
	end = int(info[3])
	if not gff_speci_vs_contig_vs_gene.has_key(speci):
		gff_speci_vs_contig_vs_gene[speci]={}
	if not gff_speci_vs_contig_vs_gene[speci].has_key(contig):
		gff_speci_vs_contig_vs_gene[speci][contig]=[]
	gff_speci_vs_contig_vs_gene[speci][contig].append([gene,start,end])
	gene_vs_verything[gene]={}
	gene_vs_verything[gene]['speci']=speci
	gene_vs_verything[gene]['contig']=contig
	gene_vs_verything[gene]['start']=start
	gene_vs_verything[gene]['end']=end
F2.close()




F2 = open("mcscanx.tandem.NoRed")
tandem={}
tandem_map={}
num=0
for each_line in F2:
	each_line = re.sub('\n','',each_line)
	info = string.split(each_line,"\t")
	num=num+1
	tandem[num]=[]
	for i in info:
		tandem[num].append(i)
		tandem_map[i]=num
F2.close()


for speci in gff_speci_vs_contig_vs_gene:
	for contig in gff_speci_vs_contig_vs_gene[speci]:
		gff_speci_vs_contig_vs_gene[speci][contig]=sorted(gff_speci_vs_contig_vs_gene[speci][contig],key=lambda e:e[1],reverse=False)


		
colline_block_all,block_id_dict_all = Parse_collinearity("mcscanx.collinearity")
colline_block_standard,block_id_dict_standard = Parse_collinearity("mcscanxh.collinearity")
Intra_overlap = Find_intra_overlap_block(colline_block_all,colline_block_standard)
colline_block_all_range,colline_block_all_range_map,intersection_all_dict=Find_self_overlap_block(colline_block_all)


saved_all=[]
saved_all_list=[]
for speci_tag in Intra_overlap:
	for rec in Intra_overlap[speci_tag]:
		saved_all.append([rec[1]])
		saved_all_list.append(rec[1])
for group in intersection_all_dict['overlap']:
	for id in group:
		if id in saved_all_list:
			saved_all.append(group)
			
saved_all = NoRed_element(saved_all)
saved_all_map = {}
for group in saved_all:
	for block_id in group:
		saved_all_map[block_id]=saved_all.index(group)


#####################################准备完成
		
Cc_gene_block_saved={}
for gene in gene_vs_verything:	
	if gene_vs_verything[gene]['speci'] == "Cc":
		output = block_search_gene(gene,gene,gene_vs_verything,colline_block_all_range,colline_block_all_range_map)
		Cc_gene_block_saved[gene]=[]
		for block_id in output:
			if not saved_all_map.has_key(block_id):
				continue
			Cc_gene_block_saved[gene].append(saved_all_map[block_id])
		Cc_gene_block_saved[gene]=list(set(Cc_gene_block_saved[gene]))
		temp_list=[]
		for index_saved in Cc_gene_block_saved[gene]:
			temp_list.append(saved_all[index_saved])
		Cc_gene_block_saved[gene] = temp_list


num=0
OUT = open("Cca_gene_block_num","w")
for gene in Cc_gene_block_saved:
	print >> OUT,gene,len(Cc_gene_block_saved[gene])
OUT.close()
		
classic_range={}
for gene in Cc_gene_block_saved:
	score={}
	score[('Ca', 'Cc')]=0
	score[('Cc', 'In')]=0
	for block_id_group in Cc_gene_block_saved[gene]:
		speci_tag=block_id_dict_all[block_id_group[0]]['speci_tag']
		for i in score:
			if speci_tag == i:
				score[speci_tag]=score[speci_tag]+1
	if not classic_range.has_key((score[('Ca', 'Cc')],score[('Cc', 'In')])):
		classic_range[(score[('Ca', 'Cc')],score[('Cc', 'In')])]=[]
	classic_range[(score[('Ca', 'Cc')],score[('Cc', 'In')])].append(gene)
		
classic_range_map={}
for i in classic_range:
	for gene in classic_range[i]:
		classic_range_map[gene]=i
		
Cc_coll_stat={}
for contig_Cc in gff_speci_vs_contig_vs_gene['Cc']:
	Cc_coll_stat[contig_Cc]=[]
	for gene in gff_speci_vs_contig_vs_gene['Cc'][contig_Cc]:
		gene = gene[0]
		Cc_coll_stat[contig_Cc].append([gene,classic_range_map[gene]])
		
num=0
OUT = open("Cc_coll_stat","w")
for contig_Cc in Cc_coll_stat:
	for gene in Cc_coll_stat[contig_Cc]:
		gene,Cc_coll = gene
		print >> OUT,gene,Cc_coll
OUT.close()


Target_contig='chr2'
string_search="Cc:chr2:1-54521787"
all_output = block_search_range(string_search,colline_block_all_range,colline_block_all_range_map)


figure_block_id=[]
for block_id in all_output:
	if block_id in saved_all_list:
		figure_block_id.append(block_id)


		


Big_block=[]
for block_id1 in figure_block_id:
	for block_id2 in figure_block_id:
		if block_id1 == block_id2:
			continue
		speci_tag1 = block_id_dict_all[block_id1]['speci_tag']
		Cc_index1 = speci_tag1.index('Cc')
		speci_tag2 = block_id_dict_all[block_id2]['speci_tag']
		Cc_index2 = speci_tag2.index('Cc')
		block_id1_info=colline_block_all_range[speci_tag1][block_id1]
		block_id2_info=colline_block_all_range[speci_tag2][block_id2]
		Cc_flag,useless = section(block_id1_info[Cc_index1][2],block_id2_info[Cc_index2][2])
		if Cc_flag:
			Big_block.append([block_id1,block_id2])
Big_block = NoRed_element(Big_block)
	
Big_block_dict={}	
for group in Big_block:
	Big_block_dict[Big_block.index(group)]={}
	group_range=[]
	group_speci=[]
	for block_id in group:
		speci_tag = block_id_dict_all[block_id]['speci_tag']
		Cc_index = speci_tag.index('Cc')
		block_id_info=colline_block_all_range[speci_tag][block_id]
		group_range.extend(block_id_info[Cc_index][2])
		group_speci.extend(list(speci_tag))
	group_speci=list(set(group_speci))
	group_range=[min(group_range),max(group_range)]
	Big_block_dict[Big_block.index(group)]['range']=group_range
	Big_block_dict[Big_block.index(group)]['coverage']=max(group_range)-min(group_range)
	Big_block_dict[Big_block.index(group)]['group_speci']=group_speci


a=[9455417,14986889]
b=[]
for start in a:
	for index in Big_block_dict:
		if Big_block_dict[index]['range'][0]==start:
			b.append(index)


OUT = open("figure_link.txt","w")
for index in b:
	group = Big_block[index]
	block_contig=[]
	for block_id in group:
		speci_tag = block_id_dict_all[block_id]['speci_tag']
		colline_block_all
		colline_block_all_range
		colline_block_all_range_map[speci_tag]
		block_contig.append(block_id_dict_all[block_id]['speci_tag'][0]+block_id_dict_all[block_id]['speci1_contig'])
		block_contig.append(block_id_dict_all[block_id]['speci_tag'][1]+block_id_dict_all[block_id]['speci2_contig'])
		[[speci1,speci1_contig,speci1_contig_range],[speci2,speci2_contig,speci2_contig_range]]=colline_block_all_range[speci_tag][block_id]
		print >> OUT,speci1+speci1_contig,speci1_contig_range[0],speci1_contig_range[1],speci2+speci2_contig,speci2_contig_range[0],speci2_contig_range[1]
	block_contig=list(set(block_contig))
OUT.close()




OUT = open("length_ratio","w")		
import math
for speci_tag in colline_block_all_range:
	for block_id in colline_block_all_range[speci_tag]:
		[[speci1,speci1_contig,speci1_contig_range],[speci2,speci2_contig,speci2_contig_range]]=colline_block_all_range[speci_tag][block_id]
		spec1_length = float(speci1_contig_range[1]-speci1_contig_range[0])
		spec2_length = float(speci2_contig_range[1]-speci2_contig_range[0])
		log_ratio = math.log(spec1_length/spec2_length,2)
		print >> OUT,speci_tag,log_ratio
OUT.close()
			
block_contig={}
for index in b:
	group = Big_block[index]
	for block_id in group:
		speci_tag = block_id_dict_all[block_id]['speci_tag']
		[[speci1,speci1_contig,speci1_contig_range],[speci2,speci2_contig,speci2_contig_range]]=colline_block_all_range[speci_tag][block_id]
		if not block_contig.has_key(speci1+speci1_contig):
			block_contig[speci1+speci1_contig]=[]
		block_contig[speci1+speci1_contig]=[min(speci1_contig_range+block_contig[speci1+speci1_contig]),max(speci1_contig_range+block_contig[speci1+speci1_contig])]
		if not block_contig.has_key(speci2+speci2_contig):
			block_contig[speci2+speci2_contig]=[]
		block_contig[speci2+speci2_contig]=[min(speci2_contig_range+block_contig[speci2+speci2_contig]),max(speci2_contig_range+block_contig[speci2+speci2_contig])]


Cc_gene_orthology={}
for gene in gene_vs_verything:	
	if gene_vs_verything[gene]['speci'] == "Cc":
		output = block_search_gene(gene,gene,gene_vs_verything,colline_block_all_range,colline_block_all_range_map)
		Cc_gene_block_saved[gene]=[]
		for block_id in output:
			if not saved_all_map.has_key(block_id):
				continue
			speci_tag = block_id_dict_all[block_id]['speci_tag']
			Cc_index = speci_tag.index('Cc')
			if gene in colline_block_all[speci_tag][block_id][Cc_index]:
				gene_index = colline_block_all[speci_tag][block_id][Cc_index].index(gene)
				if Cc_index == 1:
					ortho_gene = colline_block_all[speci_tag][block_id][0][gene_index]
				else:
					ortho_gene = colline_block_all[speci_tag][block_id][1][gene_index]
				if not Cc_gene_orthology.has_key(gene):
					Cc_gene_orthology[gene]=[]
				Cc_gene_orthology[gene].append(ortho_gene)
				Cc_gene_orthology[gene]=list(set(Cc_gene_orthology[gene]))


Cc_gene_orthology_tree={}
for Cc_gene in Cc_gene_orthology:
	Ca = 0
	In = 0
	for other_gene in Cc_gene_orthology[Cc_gene]:
		if gene_vs_verything[other_gene]['speci'] == 'Ca':
			Ca=Ca+1
		elif gene_vs_verything[other_gene]['speci'] == 'In':
			In=In+1
	if Ca >= 2 and In >= 2:
		Cc_gene_orthology_tree[Cc_gene] = Cc_gene_orthology[Cc_gene]
		Cc_gene_orthology_tree[Cc_gene].append(Cc_gene)
		print Cc_gene_orthology_tree[Cc_gene]


phb_file_dict={}
for Cc_gene in Cc_gene_orthology_tree:
	subprocess.call("rm "+Cc_gene+".faa", shell=True)
	for gene in Cc_gene_orthology_tree[Cc_gene]:
		output_file = samtools_to_fasta(gene,"all.pep.faa",Cc_gene+".faa")
	phb_file = phylogene_tree(output_file,Cc_gene)
	phb_file_dict[Cc_gene]=phb_file


				
				
				
				
all_orthology=[]
for block_id in saved_all_list:
	speci_tag = block_id_dict_all[block_id]['speci_tag']
	for i in range(0,len(colline_block_all[speci_tag][block_id][0])):
		all_orthology.append([colline_block_all[speci_tag][block_id][0][i],colline_block_all[speci_tag][block_id][1][i]])
		
all_orthology=NoRed_element(all_orthology)


OUT = open("all_orthology","w")		
for i in all_orthology:
	printer = ""
	for j in i:
		printer=printer+j+"\t"
	printer = printer.rstrip('\t')
	print >> OUT,printer
OUT.close()



from pyfaidx import Fasta
from lib.common.math.interval import merge_intervals
from lib.common.os import multiprocess_running
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation
from BCBio import GFF

def contig_range(SeqRecord_input):
    sequence = str(SeqRecord_input.seq)

    num = 1
    n_site = []
    for i in sequence:
        if i != "N":
            n_site.append((num, num))
        num = num + 1
    contig_range_output = merge_intervals(n_site, True)

    SeqRecord_input.features = []

    num = 0
    for contig_range_tmp in contig_range_output:
        qualifiers = {
            "source": "ReGetContig",
            "ID": "%s_%d" % (SeqRecord_input.id, num)
        }
        top_feature = SeqFeature(FeatureLocation(contig_range_tmp[0] - 1, contig_range_tmp[1]),
                                    type="contig", qualifiers=qualifiers)
        SeqRecord_input.features.append(top_feature)
        num = num + 1

    return SeqRecord_input


def ReGetContig_main(args):

    scaff_dir = Fasta(args.scaff_file)

    SeqRecord_list = []

    for scaff in scaff_dir:
        scaff_seq = Seq(str(scaff))
        SeqRecord_tmp = SeqRecord(scaff_seq, scaff.name)
        SeqRecord_list.append((SeqRecord_tmp,))

    cmd_result = multiprocess_running(
        contig_range, SeqRecord_list, args.threads, True)

    with open(args.output_gff3, 'w') as f:
        for i in cmd_result:
            GFF.write([cmd_result[i]['output']], f)

    with open(args.output_fasta, 'w') as f:
        for i in cmd_result:
            SeqRecord_tmp = cmd_result[i]['output']

            record_feature = SeqRecord_tmp.features
            for contig_feature in record_feature:
                start = int(contig_feature.location.start)
                end = int(contig_feature.location.end)
                ID = contig_feature.qualifiers['ID'][0]
                contig_seq = SeqRecord_tmp.seq[start:end]

                contig_record = SeqRecord(contig_seq, id=ID,
                                            description="%s:%d-%d" % (SeqRecord_tmp.id, start + 1, end))
                f.write(contig_record.format("fasta"))
                # SeqIO.write([contig_record], args.output_fasta, "fasta")

if __name__ == "__main__":
    class abc():
        pass

    args=abc()

    args.scaff_file = "/lustre/home/xuyuxing/Database/Cuscuta/Csp/Csp.fa"
    args.threads = 5
    args.output_gff3 = "/lustre/home/xuyuxing/Work/Csp/HiC/contig.gff3"
    args.output_fasta = "/lustre/home/xuyuxing/Work/Csp/HiC/contig.fasta"

    ReGetContig_main(args)



def draw_tree_to_file(tree, tree_style, output_file):
    from pyvirtualdisplay import Display

    display = Display(visible=False, color_depth=24)
    display.start()

    print(output_file)
    tree.render(output_file, tree_style=tree_style)
    # t.show(tree_style=ts)

    display.stop()



"""
This file work for Mercator output

Mercator4 v2.0 x (July 2019)

https://www.plabipd.de/portal/web/guest/mercator4
"""
from Bio.Phylo.BaseTree import Tree, Clade
from lib.common.fileIO import tsv_file_dict_parse
import re
from collections import OrderedDict
import copy


def get_parent_id(bincode):
    parent_id = ".".join(bincode.split(".")[:-1])
    if parent_id == '':
        return None
    else:
        return parent_id


def get_ancestors_id_list(bincode):
    ancestors_id_list = []
    lastcode = bincode
    flag = True
    while flag:
        parent_id = get_parent_id(lastcode)
        if parent_id:
            ancestors_id_list.append(parent_id)
            lastcode = parent_id
        else:
            flag = False
    return sorted(ancestors_id_list)


class MapmanFunction(Clade):
    def __init__(
        self,
        bin_code,
        name=None,
    ):
        super(MapmanFunction, self).__init__(
            branch_length=None,
            name=name,
            clades=None,
            confidence=None,
            color=None,
            width=None,
        )

        self.bin_code = bin_code
        self.depth = len(get_ancestors_id_list(bin_code)) + 1

    def __str__(self):
        return '%s: %s (%d)' % (self.bin_code, self.name, self.depth)

    __repr__ = __str__


class MapmanTree(Tree):
    def __init__(self, mercator_results_file):
        mercator_info_dict = tsv_file_dict_parse(mercator_results_file)

        # add node
        mercator_dict = OrderedDict()
        for i in mercator_info_dict:
            bin_code = re.sub("'", "", mercator_info_dict[i]['BINCODE'])
            name = re.sub(
                "'", "", mercator_info_dict[i]['NAME'].split(".")[-1])
            # description = re.sub("'","",mercator_info_dict[i]['DESCRIPTION'])
            mercator_dict[bin_code] = MapmanFunction(bin_code, name=name)

        # add clade relationship
        top_bin_code = []
        for bin_code in mercator_dict:
            if get_parent_id(bin_code):
                parent_id = get_parent_id(bin_code)
                if mercator_dict[parent_id].clades is None:
                    mercator_dict[parent_id].clades = []
                mercator_dict[parent_id].clades.append(mercator_dict[bin_code])
            else:
                top_bin_code.append(mercator_dict[bin_code])

        root_clade = Clade(clades=top_bin_code, name='root')
        root_clade.bin_code = "0"
        root_clade.depth = 0

        super(MapmanTree, self).__init__(
            root=root_clade,
            rooted=True,
            id=None,
            name=None
        )

    def load_gene_annotation(self, mercator_results_file, sp_id='default_sp_id'):
        mercator_info_dict = tsv_file_dict_parse(mercator_results_file)
        gene_bincode_map = {}
        for i in mercator_info_dict:
            bin_code = re.sub("'", "", mercator_info_dict[i]['BINCODE'])
            gene_id = re.sub(
                "'", "", mercator_info_dict[i]['IDENTIFIER'])
            if bin_code not in gene_bincode_map:
                gene_bincode_map[bin_code] = []
            gene_bincode_map[bin_code].append(gene_id)

        for leaf in self.get_terminals():
            if not hasattr(leaf, 'genes'):
                leaf.genes = {}
            leaf.genes[sp_id] = []

        for leaf in self.get_terminals():
            gene_list_tmp = list(set(leaf.genes[sp_id] + gene_bincode_map[leaf.bin_code]))
            try:
                gene_list_tmp.remove('')
            except:
                pass

            leaf.genes[sp_id] = gene_list_tmp

        self.sp_list = list(set(leaf.genes.keys()))

    def get_genes(self, bin_code, sp_id='default_sp_id'):
        target_clade = None
        for clade in self.find_clades(order='preorder'):
            if clade.bin_code == bin_code:
                target_clade = clade
                break

        gene_list = []
        if target_clade:
            for leaf in target_clade.get_terminals():
                gene_list += leaf.genes[sp_id]

        gene_list = list(set(gene_list))
        gene_list.remove('')

        return gene_list


if __name__ == "__main__":
    mercator_results_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Dendrobium_catenatum/T906689N0.gene_model.protein.Mercator.results.txt'

    mtree = MapmanTree(mercator_results_file)

    # mtree can work as Bio Tree and Clade

    Dca_mercator_results_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Dendrobium_catenatum/T906689N0.gene_model.protein.Mercator.results.txt'
    Gel_mercator_results_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Gastrodia_elata/T91201N0.gene_model.protein.Mercator.results.txt'
    Cau_mercator_results_file = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Cuscuta_australis/T267555N0.gene_model.protein.Mercator.results.txt'

    mtree.load_gene_annotation(Dca_mercator_results_file, 'Dca')
    mtree.load_gene_annotation(Gel_mercator_results_file, 'Gel')
    mtree.load_gene_annotation(Cau_mercator_results_file, 'Cau')


import networkx as nx
from itertools import combinations
from lib.common.util import dict_key_value_transpose


def merge_same_element_set_with_group_id_by_graphs(input_dict):
    """
    merge sets which have at least one same element
    based on Connected component in graphs
    :param input_dict:
            input_dict = {
                "a" : [1,2,3,4,5],
                "b" : [10,2,30,40,50],
                "c" : [100,200,300,40,500],
                "d" : [900,1000]
                }
    :return: output_list = [(a,b,c),(d)]
    """
    G = nx.Graph()
    G.add_nodes_from(input_dict.keys())
    input_hash = dict_key_value_transpose(input_dict)
    for element in input_hash:
        if len(input_hash[element]) > 1:
            G.add_edges_from(combinations(input_hash[element], 2))

    output_list = []
    for sub_graph in (G.subgraph(c) for c in nx.connected_components(G)):
        output_list.append(tuple(list(sub_graph.nodes)))

    output_dir = {}
    for i in output_list:
        element_list = []
        for j in i:
            element_list.extend(input_dict[j])
        output_dir[i] = list(set(element_list))

    return output_dir


def find_longest_path(G, nodeA, nodeB):
    """
    heaviest_path = max((path for path in nx.all_simple_paths(G, nodeA, nodeB)),
                        key=lambda path: get_weight(path))

    """

    """
    DG = nx.DiGraph()
    DG.add_edge('S', 'a', weight=-1)
    DG.add_edge('a', 'b', weight=-1)
    DG.add_edge('a', 'c', weight=-2)
    DG.add_edge('b', 'd', weight=-1)
    DG.add_edge('b', 'e', weight=-2)
    DG.add_edge('c', 'e', weight=-3)
    DG.add_edge('c', 'f', weight=-2)
    DG.add_edge('d', 'T', weight=-1)
    DG.add_edge('e', 'T', weight=-1)
    DG.add_edge('f', 'T', weight=-1)
    
    p2 = nx.johnson(G, weight='weight')
    print('johnson: {0}'.format(p2['S']['T']))
    """

    pass
